@@ -59,7 +59,7 @@ class ImportTab(qt4.QWidget):
         should be attempted."""
         pass
 
-    def doImport(self, filename, linked, encoding, prefix, suffix):
+    def doImport(self, filename, linked, encoding, prefix, suffix, tags):
         """Do the import iteself."""
         pass
 
@@ -112,11 +112,11 @@ class ImportTabStandard(ImportTab):
 
             self.previewedit.setPlainText(text)
             return True
-        except (UnicodeError, IOError):
+        except (UnicodeError, EnvironmentError):
             self.previewedit.setPlainText('')
             return False
 
-    def doImport(self, doc, filename, linked, encoding, prefix, suffix):
+    def doImport(self, doc, filename, linked, encoding, prefix, suffix, tags):
         """Standard Veusz importing."""
 
         # convert controls to values
@@ -124,14 +124,20 @@ class ImportTabStandard(ImportTab):
         useblocks = self.blockcheckbox.isChecked()
         ignoretext = self.ignoretextcheckbox.isChecked()
 
+        params = document.ImportParamsSimple(
+            descriptor=descriptor,
+            filename=filename,
+            useblocks=useblocks,
+            linked=linked,
+            prefix=prefix, suffix=suffix,
+            tags=tags,
+            ignoretext=ignoretext,
+            encoding=encoding,
+            )
+
         try:
             # construct operation. this checks the descriptor.
-            op = document.OperationDataImport(descriptor, filename=filename,
-                                              useblocks=useblocks, 
-                                              linked=linked,
-                                              prefix=prefix, suffix=suffix,
-                                              ignoretext=ignoretext,
-                                              encoding=encoding)
+            op = document.OperationDataImport(params)
 
         except document.DescriptorError:
             qt4.QMessageBox.warning(self, "Veusz",
@@ -139,19 +145,19 @@ class ImportTabStandard(ImportTab):
             return
 
         # actually import the data
-        dsnames = doc.applyOperation(op)
-        
+        doc.applyOperation(op)
+
         # tell the user what happened
         # failures in conversion
         lines = []
-        for var, count in op.simpleread.getInvalidConversions().iteritems():
+        for var, count in op.outinvalids.iteritems():
             if count != 0:
                 lines.append('%i conversions failed for dataset "%s"' %
                              (count, var))
         if len(lines) != 0:
             lines.append('')
-            
-        lines += self.dialog.retnDatasetInfo(dsnames, linked, filename)
+
+        lines += self.dialog.retnDatasetInfo(op.outdatasets, linked, filename)
 
         self.previewedit.setPlainText( '\n'.join(lines) )
 
@@ -196,7 +202,7 @@ class ImportTabCSV(ImportTab):
         self.csvblanksdatacheck.setChecked(False)
         self.csvnumfmtcombo.setCurrentIndex(0)
         self.csvdatefmtcombo.setEditText(
-            document.ParamsCSV.defaults['dateformat'])
+            document.ImportParamsCSV.defaults['dateformat'])
         self.csvheadermodecombo.setCurrentIndex(0)
 
     def slotHelp(self):
@@ -254,7 +260,7 @@ class ImportTabCSV(ImportTab):
                 pass
             numrows = len(rows)
 
-        except (IOError, UnicodeError, csv.Error):
+        except (EnvironmentError, UnicodeError, csv.Error):
             return False
 
         # fill up table
@@ -268,7 +274,7 @@ class ImportTabCSV(ImportTab):
 
         return True
 
-    def doImport(self, doc, filename, linked, encoding, prefix, suffix):
+    def doImport(self, doc, filename, linked, encoding, prefix, suffix, tags):
         """Import from CSV file."""
 
         # get various values
@@ -291,8 +297,8 @@ class ImportTabCSV(ImportTab):
             self.csvheadermodecombo.currentIndex()]
 
         # create import parameters and operation objects
-        params = document.ParamsCSV(
-            filename,
+        params = document.ImportParamsCSV(
+            filename=filename,
             readrows=inrows,
             encoding=encoding,
             delimiter=delimiter,
@@ -303,15 +309,17 @@ class ImportTabCSV(ImportTab):
             numericlocale=numericlocale,
             dateformat=dateformat,
             headermode=headermode,
-            dsprefix=prefix, dssuffix=suffix
+            prefix=prefix, suffix=suffix,
+            tags=tags,
+            linked=linked,
             )
-        op = document.OperationDataImportCSV(params, linked=linked)
+        op = document.OperationDataImportCSV(params)
 
         # actually import the data
-        dsnames = doc.applyOperation(op)
+        doc.applyOperation(op)
 
         # update output, showing what datasets were imported
-        lines = self.dialog.retnDatasetInfo(dsnames, linked, filename)
+        lines = self.dialog.retnDatasetInfo(op.outdatasets, linked, filename)
 
         t = self.previewtablecsv
         t.verticalHeader().hide()
@@ -365,11 +373,11 @@ class ImportTab2D(ImportTab):
             self.twod_previewedit.setPlainText(text)
             return True
 
-        except (UnicodeError, IOError):
+        except (UnicodeError, EnvironmentError):
             self.twod_previewedit.setPlainText('')
             return False
 
-    def doImport(self, doc, filename, linked, encoding, prefix, suffix):
+    def doImport(self, doc, filename, linked, encoding, prefix, suffix, tags):
         """Import from 2D file."""
 
         # this really needs improvement...
@@ -412,26 +420,32 @@ class ImportTab2D(ImportTab):
         transpose = self.twod_transposecheck.isChecked()
 
         # loop over datasets and read...
+        params = document.ImportParams2D(
+            datasetnames=datasets,
+            filename=filename,
+            xrange=rangex, yrange=rangey,
+            invertrows=invertrows,
+            invertcols=invertcols,
+            transpose=transpose,
+            prefix=prefix, suffix=suffix,
+            tags=tags,
+            linked=linked,
+            encoding=encoding
+            )
+
         try:
-            op = document.OperationDataImport2D(datasets, filename=filename,
-                                                xrange=rangex, yrange=rangey,
-                                                invertrows=invertrows,
-                                                invertcols=invertcols,
-                                                transpose=transpose,
-                                                prefix=prefix, suffix=suffix,
-                                                linked=linked,
-                                                encoding=encoding)
-            readds = doc.applyOperation(op)
+            op = document.OperationDataImport2D(params)
+            doc.applyOperation(op)
 
             output = ['Successfully read datasets:']
-            for ds in readds:
+            for ds in op.outdatasets:
                 output.append(' %s' % doc.data[ds].description(
                         showlinked=False))
             
             output = '\n'.join(output)
         except document.Read2DError, e:
             output = 'Error importing datasets:\n %s' % str(e)
-                
+
         # show status in preview box
         self.twod_previewedit.setPlainText(output)
 
@@ -486,9 +500,9 @@ class ImportTabFITS(ImportTab):
             line = ifile.readline()
             # is this a hack?
             if line.find('SIMPLE  =                    T') == -1:
-                raise IOError
+                raise EnvironmentError
             ifile.close()
-        except IOError:
+        except EnvironmentError:
             self.clearFITSView()
             return False
 
@@ -596,7 +610,7 @@ class ImportTabFITS(ImportTab):
             return True
         return False
 
-    def doImport(self, doc, filename, linked, encoding, prefix, suffix):
+    def doImport(self, doc, filename, linked, encoding, prefix, suffix, tags):
         """Import fits file."""
         
         item = self.fitshdulist.selectedItems()[0]
@@ -624,12 +638,19 @@ class ImportTabFITS(ImportTab):
             cols = [None]*4
 
         # construct operation to import fits
-        op = document.OperationDataImportFITS(name, filename, hdunum,
-                                              datacol=cols[0],
-                                              symerrcol=cols[1],
-                                              poserrcol=cols[2],
-                                              negerrcol=cols[3],
-                                              linked=linked)
+        params = document.ImportParamsFITS(
+            dsname=name,
+            filename=filename,
+            hdu=hdunum,
+            datacol=cols[0],
+            symerrcol=cols[1],
+            poserrcol=cols[2],
+            negerrcol=cols[3],
+            tags=tags,
+            linked=linked,
+            )
+
+        op = document.OperationDataImportFITS(params)
 
         # actually do the import
         doc.applyOperation(op)
@@ -752,7 +773,7 @@ class ImportTabPlugins(ImportTab):
             try:
                 f = open(filename, 'r')
                 f.close()
-            except IOError:
+            except EnvironmentError:
                 self.pluginPreview.setPlainText('')
                 return False
 
@@ -773,27 +794,35 @@ class ImportTabPlugins(ImportTab):
         self.pluginPreview.setPlainText(text)
         return bool(ok)
 
-    def doImport(self, doc, filename, linked, encoding, prefix, suffix):
+    def doImport(self, doc, filename, linked, encoding, prefix, suffix, tags):
         """Import using plugin."""
         
-        params = self.getPluginFields()
+        fields = self.getPluginFields()
         plugin = unicode(self.pluginType.currentText())
-        op = document.OperationDataImportPlugin(
-            plugin, filename, linked=linked, encoding=encoding,
-            prefix=prefix, suffix=suffix, **params)
+
+        params = document.ImportParamsPlugin(
+            plugin=plugin,
+            filename=filename,
+            linked=linked, encoding=encoding,
+            prefix=prefix, suffix=suffix,
+            tags=tags,
+            **fields)
+
+        op = document.OperationDataImportPlugin(params)
         try:
-            datasets, customs = doc.applyOperation(op)
+            doc.applyOperation(op)
         except plugins.ImportPluginException, ex:
             self.pluginPreview.setPlainText( unicode(ex) )
             return
 
         out = ['Imported data for datasets:']
-        for ds in datasets:
+        for ds in op.outdatasets:
             out.append( doc.data[ds].description(showlinked=False) )
-        if customs:
+        if op.outcustoms:
             out.append('')
             out.append('Set custom definitions:')
-            out += customs
+            # format custom definitions
+            out += ['%s %s=%s' % tuple(c) for c in op.outcustoms]
 
         self.pluginPreview.setPlainText('\n'.join(out))
 
@@ -999,11 +1028,13 @@ class ImportDialog(VeuszDialog):
         # import according to tab selected
         importtab = self.methodtab.currentWidget()
         prefix, suffix = self.getPrefixSuffix(filename)
+        tags = unicode(self.tagcombo.currentText()).split()
+
         try:
             qt4.QApplication.setOverrideCursor( qt4.QCursor(qt4.Qt.WaitCursor) )
             self.document.suspendUpdates()
             importtab.doImport(self.document, filename, linked, encoding,
-                               prefix, suffix)
+                               prefix, suffix, tags)
             qt4.QApplication.restoreOverrideCursor()
         except Exception:
             qt4.QApplication.restoreOverrideCursor()
