@@ -31,7 +31,10 @@ import veusz.qtall as qt4
 import veusz.setting as setting
 import veusz.embed as embed
 import veusz.plugins as plugins
+import veusz.utils as utils
 
+import linked
+import importparams
 import datasets
 import operations
 import dataset_histo
@@ -54,6 +57,7 @@ class CommandInterface(qt4.QObject):
         'Get',
         'GetChildren',
         'GetData',
+        'GetDataType',
         'GetDatasets',
         'ImportFITSFile',
         'ImportFile',
@@ -77,11 +81,13 @@ class CommandInterface(qt4.QObject):
         'SetData2DExpression',
         'SetData2DExpressionXYZ',
         'SetData2DXYFunc',
+        'SetDataDateTime',
         'SetDataExpression',
         'SetDataRange',
         'SetDataText',
         'SettingType',
         'SetVerbose',
+        'TagDatasets',
         'To',
         'WidgetType',
         )
@@ -121,7 +127,7 @@ class CommandInterface(qt4.QObject):
                 opened = open(fname)
                 opened.close()
                 return fname
-            except IOError:
+            except EnvironmentError:
                 pass
         return filename
 
@@ -129,17 +135,14 @@ class CommandInterface(qt4.QObject):
         """Specify whether we want verbose output after operations."""
         self.verbose = v
 
-    def Add(self, widgettype, *args, **args_opt):
+    def Add(self, widgettype, **args_opt):
         """Add a widget to the widget with the type given.
 
-        optional argument:
+        optional arguments:
           widget: widget path to place widget in
-
-        The optional arguments are sent to construct the widget.
-
-        If autoadd is True (the default), then any sub widgets
-        associated with the widgets are added automatically (e.g. axes
-        for graph widgets).
+          autoadd: if True (default), any subwidgets associated with
+            widget are created automatically
+          setting names, e.g. leftMargin='1cm', Border_color='red'
         """
 
         at = self.currentwidget
@@ -147,7 +150,7 @@ class CommandInterface(qt4.QObject):
             at = self.document.resolve(self.currentwidget, args_opt['widget'])
             del args_opt['widget']
 
-        op = operations.OperationWidgetAdd(at, widgettype, *args, **args_opt)
+        op = operations.OperationWidgetAdd(at, widgettype, **args_opt)
         w = self.document.applyOperation(op)
 
         if self.verbose:
@@ -388,12 +391,25 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
  
         if self.verbose:
-            print "Set variable '%s':" % name
+            print "Set dataset '%s':" % name
             print " Values = %s" % str( data.data )
             print " Symmetric errors = %s" % str( data.serr )
             print " Negative errors = %s" % str( data.nerr )
             print " Positive errors = %s" % str( data.perr )
 
+    def SetDataDateTime(self, name, vals):
+        """Set datetime dataset to be values given.
+        vals is a list of python datetime objects
+        """
+        v = [utils.datetimeToFloat(x) for x in vals]
+        ds = datasets.DatasetDateTime(v)
+        op = operations.OperationDatasetSet(name, ds)
+        self.document.applyOperation(op)
+
+        if self.verbose:
+            print "Set dataset '%s':" % name
+            print " Values = %s" % str(ds.data)
+
     def SetDataExpression(self, name, val, symerr=None, negerr=None, poserr=None,
                           linked=False, parametric=None):
         """Create a dataset based on text expressions.
@@ -419,7 +435,7 @@ class CommandInterface(qt4.QObject):
         data = self.document.applyOperation(op)
         
         if self.verbose:
-            print "Set variable '%s' based on expression:" % name
+            print "Set dataset '%s' based on expression:" % name
             print " Values = %s" % str( data.data )
             print " Symmetric errors = %s" % str( data.serr )
             print " Negative errors = %s" % str( data.nerr )
@@ -444,7 +460,7 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
         
         if self.verbose:
-            print "Set variable '%s' based on range:" % name
+            print "Set dataset '%s' based on range:" % name
             print " Number of steps = %i" % numsteps
             print " Range of data = %s" % repr(val)
             print " Range of symmetric error = %s" % repr(symerr)
@@ -532,28 +548,30 @@ class CommandInterface(qt4.QObject):
 
         if self.verbose:
             print "Set text dataset '%s'" % name
-            print " Values = %s" % str(data.data)
+            print " Values = %s" % unicode(data.data)
 
     def GetData(self, name):
         """Return the data with the name.
 
-        Returns a tuple containing:
+        For a 1D dataset, returns a tuple (None if not defined)
             (data, serr, nerr, perr)
-        if 'name' is a Dataset, and 
-            data
-        if 'name' is a DatasetText, where data is a list.
-
-        Values not defined are set to None
+        For a 2D dataset, returns
+            (data, xrange, yrange)
+        For a text dataset, return a list of text
+        For a date dataset, return a list of python datetime objects
 
         Return copies, so that the original data can't be indirectly modified
         """
 
         d = self.document.getData(name)
-        data = serr = nerr = perr = None
-        if isinstance(d, datasets.DatasetText):
+        if d.displaytype == 'text':
             return d.data[:]
+        elif d.displaytype == 'date':
+            return [utils.floatToDateTime(x) for x in d.data]
+        elif d.dimensions == 2:
+            return (d.data.copy(), d.xrange, d.yrange)
         else:
-
+            data = serr = nerr = perr = None
             if d.data is not None:
                 data = d.data.copy()
             if d.serr is not None:
@@ -565,7 +583,29 @@ class CommandInterface(qt4.QObject):
 
             return (data, serr, nerr, perr)
 
-    def ImportString(self, descriptor, string, useblocks=False):
+    def GetDataType(self, name):
+        """Return the type of dataset. Returns None if no dataset.
+
+        For a 1D dataset, returns '1d'
+        For a 2D dataset, returns '2d'
+        For a text dataset, returns 'text'
+        For a datetime dataset, returns 'datetime'
+        """
+
+        try:
+            d = self.document.getData(name)
+        except KeyError:
+            return None
+        if d.displaytype == 'text':
+            return 'text'
+        elif d.displaytype == 'date':
+            return 'datetime'
+        elif d.dimensions == 2:
+            return '2d'
+        else:
+            return '1d'
+
+    def ImportString(self, descriptor, dstring, useblocks=False):
         """Read data from the string using a descriptor.
 
         If useblocks is set, then blank lines or the word 'no' are used
@@ -578,18 +618,21 @@ class CommandInterface(qt4.QObject):
          converting the data
         """
 
-        op = operations.OperationDataImport(descriptor, datastr=string, useblocks=useblocks)
-        dsnames = self.document.applyOperation(op)
-        errors = op.simpleread.getInvalidConversions()
-            
+        params = importparams.ImportParamsSimple(
+            descriptor=descriptor,
+            datastr=dstring,
+            useblocks=useblocks)
+        op = operations.OperationDataImport(params)
+        self.document.applyOperation(op)
+
         if self.verbose:
-            print "Imported datasets %s" % (' '.join(dsnames),)
-            for name, num in errors.iteritems():
+            print "Imported datasets %s" % (' '.join(op.outdatasets),)
+            for name, num in op.outinvalids.iteritems():
                 print "%i errors encountered reading dataset %s" % (num, name)
 
-        return (dsnames, errors)
+        return (op.outdatasets, op.outinvalids)
 
-    def ImportString2D(self, datasetnames, string, xrange=None, yrange=None,
+    def ImportString2D(self, datasetnames, dstring, xrange=None, yrange=None,
                        invertrows=None, invertcols=None, transpose=None):
         """Read two dimensional data from the string specified.
         datasetnames is a list of datasets to read from the string or a single
@@ -607,10 +650,12 @@ class CommandInterface(qt4.QObject):
         if type(datasetnames) in (str, unicode):
             datasetnames = [datasetnames]
 
-        op = operations.OperationDataImport2D(
-            datasetnames, datastr=string, xrange=xrange,
+        params = importparams.ImportParams2D(
+            datasetnames=datasetnames,
+            datastr=dstring, xrange=xrange,
             yrange=yrange, invertrows=invertrows,
             invertcols=invertcols, transpose=transpose)
+        op = operations.OperationDataImport2D(params)
         self.document.applyOperation(op)
         if self.verbose:
             print "Imported datasets %s" % (', '.join(datasetnames))
@@ -643,12 +688,14 @@ class CommandInterface(qt4.QObject):
         if type(datasetnames) in (str, unicode):
             datasetnames = [datasetnames]
 
-        op = operations.OperationDataImport2D(
-            datasetnames, filename=realfilename, xrange=xrange,
+        params = importparams.ImportParams2D(
+            datasetnames=datasetnames, 
+            filename=realfilename, xrange=xrange,
             yrange=yrange, invertrows=invertrows,
             invertcols=invertcols, transpose=transpose,
             prefix=prefix, suffix=suffix,
             linked=linked)
+        op = operations.OperationDataImport2D(params)
         self.document.applyOperation(op)
         if self.verbose:
             print "Imported datasets %s" % (', '.join(datasetnames))
@@ -675,20 +722,20 @@ class CommandInterface(qt4.QObject):
 
         realfilename = self.findFileOnImportPath(filename)
 
-        op = operations.OperationDataImport(
-            descriptor, filename=realfilename,
+        params = importparams.ImportParamsSimple(
+            descriptor=descriptor, filename=realfilename,
             useblocks=useblocks, linked=linked,
             prefix=prefix, suffix=suffix,
             ignoretext=ignoretext)
-        dsnames = self.document.applyOperation(op)
-        errors = op.simpleread.getInvalidConversions()
-            
+        op = operations.OperationDataImport(params)
+        self.document.applyOperation(op)
+
         if self.verbose:
-            print "Imported datasets %s" % (' '.join(dsnames),)
-            for name, num in errors.iteritems():
+            print "Imported datasets %s" % (' '.join(op.outdatasets),)
+            for name, num in op.outinvalids.iteritems():
                 print "%i errors encountered reading dataset %s" % (num, name)
 
-        return (dsnames, errors)
+        return (op.outdatasets, op.outinvalids)
 
     def ImportFileCSV(self, filename,
                       readrows=False,
@@ -732,23 +779,24 @@ class CommandInterface(qt4.QObject):
         # lookup filename
         realfilename = self.findFileOnImportPath(filename)
 
-        params = readcsv.ParamsCSV(
-            realfilename, readrows=readrows,
+        params = importparams.ImportParamsCSV(
+            filename=realfilename, readrows=readrows,
             delimiter=delimiter, textdelimiter=textdelimiter,
             encoding=encoding,
             headerignore=headerignore, rowsignore=rowsignore,
             blanksaredata=blanksaredata,
             numericlocale=numericlocale, dateformat=dateformat,
             headermode=headermode,
-            dsprefix=dsprefix, dssuffix=dssuffix,
+            prefix=dsprefix, suffix=dssuffix,
+            linked=linked,
             )
-        op = operations.OperationDataImportCSV(params, linked=linked)
-        dsnames = self.document.applyOperation(op)
-            
-        if self.verbose:
-            print "Imported datasets %s" % (' '.join(dsnames),)
+        op = operations.OperationDataImportCSV(params)
+        self.document.applyOperation(op)
 
-        return dsnames
+        if self.verbose:
+            print "Imported datasets %s" % (' '.join(op.outdatasets),)
+
+        return op.outdatasets
 
     def ImportFITSFile(self, dsname, filename, hdu,
                        datacol = None, symerrcol = None,
@@ -769,12 +817,12 @@ class CommandInterface(qt4.QObject):
 
         # lookup filename
         realfilename = self.findFileOnImportPath(filename)
-
-        op = operations.OperationDataImportFITS(
-            dsname, realfilename, hdu,
+        params = importparams.ImportParamsFITS(
+            dsname=dsname, filename=realfilename, hdu=hdu,
             datacol=datacol, symerrcol=symerrcol,
             poserrcol=poserrcol, negerrcol=negerrcol,
             linked=linked)
+        op = operations.OperationDataImportFITS(params)
         self.document.applyOperation(op)
 
     def ImportFilePlugin(self, plugin, filename, **args):
@@ -786,17 +834,22 @@ class CommandInterface(qt4.QObject):
         linked: link import to file (default False)
         encoding: file encoding (may not be used, default 'utf_8')
         plus arguments to plugin
+
+        returns: list of imported datasets, list of imported customs
         """
 
         realfilename = self.findFileOnImportPath(filename)
-        op = operations.OperationDataImportPlugin(plugin, realfilename,
-                                                  **args)
+        params = importparams.ImportParamsPlugin(
+            plugin=plugin, filename=realfilename, **args)
+
+        op = operations.OperationDataImportPlugin(params)
         try:
             self.document.applyOperation(op)
         except:
             self.document.log("Error in plugin %s" % plugin)
             exc =  ''.join(traceback.format_exc())
             self.document.log(exc)
+        return op.outdatasets, op.outcustoms
 
     def ReloadData(self):
         """Reload any linked datasets.
@@ -828,7 +881,7 @@ class CommandInterface(qt4.QObject):
             
     def Export(self, filename, color=True, page=0, dpi=100,
                antialias=True, quality=85, backcolor='#ffffff00',
-               pdfdpi=150):
+               pdfdpi=150, svgtextastext=False):
         """Export plot to filename.
 
         color is True or False if color is requested in output file
@@ -839,12 +892,13 @@ class CommandInterface(qt4.QObject):
         backcolor is the background color for bitmap files, which is a name or
          a #RRGGBBAA value (red, green, blue, alpha)
         pdfdpi is the dpi to use when exporting eps or pdf files
+        svgtextastext: write text in SVG as text, rather than curves
         """
         
         e = export.Export(self.document, filename, page, color=color,
                           bitmapdpi=dpi, antialias=antialias,
                           quality=quality, backcolor=backcolor,
-                          pdfdpi=pdfdpi)
+                          pdfdpi=pdfdpi, svgtextastext=svgtextastext)
         e.export()
 
     def Rename(self, widget, newname):
@@ -915,3 +969,12 @@ class CommandInterface(qt4.QObject):
 
         setn = self.currentwidget.prefLookup(path)
         return setn.typename
+
+    def TagDatasets(self, tag, datasets):
+        """Apply tag to list of datasets."""
+        op = operations.OperationDataTag(tag, datasets)
+        self.document.applyOperation(op)
+
+        if self.verbose:
+            print "Applied tag %s to datasets %s" % (
+                tag, ' '.join(datasets))
