@@ -24,10 +24,6 @@ from itertools import izip
 
 import numpy as N
 
-import simpleread
-import operations
-import readcsv
-
 import veusz.qtall as qt4
 import veusz.utils as utils
 import veusz.setting as setting
@@ -87,13 +83,10 @@ def generateValidDatasetParts(*datasets):
     invalid = datasets[0].invalidDataPoints()
     minlen = invalid.shape[0]
     for ds in datasets[1:]:
-        try:
+        if isinstance(ds, DatasetBase) and not ds.empty():
             nextinvalid = ds.invalidDataPoints()
             minlen = min(nextinvalid.shape[0], minlen)
             invalid = N.logical_or(invalid[:minlen], nextinvalid[:minlen])
-        except AttributeError:
-            # if not a dataset
-            pass
 
     # get indexes of invalid points
     indexes = invalid.nonzero()[0].tolist()
@@ -111,7 +104,8 @@ def generateValidDatasetParts(*datasets):
         if index != lastindex:
             retn = []
             for ds in datasets:
-                if ds is not None:
+                if ds is not None and (not isinstance(ds, DatasetBase) or
+                                       not ds.empty()):
                     retn.append( ds[lastindex:index] )
                 else:
                     retn.append( None )
@@ -125,305 +119,6 @@ def datasetNameToDescriptorName(name):
     else:
         return '`%s`' % name
 
-class LinkedFileBase(object):
-    """A base class for linked files containing common routines."""
-
-    # filename is member
-
-    def saveToFile(self, fileobj, relpath=None):
-        '''Save the link to the document file.'''
-        pass
-
-    def _getSaveFilename(self, relpath):
-        """Get filename to write to save file.
-        If relpath is a string, write relative to path given
-        """
-        if relpath:
-            return utils.relpath(self.filename, relpath)
-        else:
-            return self.filename
-
-    def reloadLinks(self, document):
-        '''Reload datasets linked to this file.'''
-        pass
-
-    def _deleteLinkedDatasets(self, document):
-        """Delete linked datasets from document linking to self."""
-
-        for name, ds in document.data.items():
-            if ds.linked == self:
-                document.deleteData(name)
-
-    def _moveReadDatasets(self, tempdoc, document):
-        """Move datasets from tempdoc to document if they do not exist
-        in the destination."""
-
-        read = []
-        for name, ds in tempdoc.data.items():
-            if name not in document.data:
-                read.append(name)
-                document.setData(name, ds)
-                ds.document = document
-                ds.linked = self
-        return read
-
-    def _reloadViaOperation(self, document, op):
-        """Reload links using a supplied operation, op."""
-        tempdoc = document.__class__()
-
-        try:
-            tempdoc.applyOperation(op)
-        except Exception, ex:
-            # if something breaks, record an error and return nothing
-            document.log(unicode(ex))
-
-            # find datasets which are linked using this link object
-            # return errors for them
-            errors = dict([(name, 1) for name, ds in document.data.iteritems()
-                           if ds.linked is self])
-            return ([], errors)            
-            
-        # delete datasets which are linked and imported here
-        self._deleteLinkedDatasets(document)
-        # move datasets into document
-        read = self._moveReadDatasets(tempdoc, document)
-
-        # return zero errors
-        errors = dict( [(ds, 0) for ds in read] )
-
-        return (read, errors)
-
-class LinkedFile(LinkedFileBase):
-    '''Instead of reading data from a string, data can be read from
-    a "linked file". This means the same document can be reloaded, and
-    the data would be reread from the file.
-
-    This class is used to store a link filename with the descriptor
-    '''
-
-    def __init__(self, filename, descriptor, useblocks=False,
-                 prefix='', suffix='', ignoretext=False,
-                 encoding='utf_8'):
-        '''Set up the linked file with the descriptor given.'''
-        self.filename = filename
-        self.descriptor = descriptor
-        self.useblocks = useblocks
-        self.prefix = prefix
-        self.suffix = suffix
-        self.ignoretext = ignoretext
-        self.encoding = encoding
-
-    def saveToFile(self, fileobj, relpath=None):
-        '''Save the link to the document file.
-        If relpath is set, save links relative to path given
-        '''
-
-        params = [ repr(self._getSaveFilename(relpath)),
-                   repr(self.descriptor),
-                   'linked=True',
-                   'ignoretext=' + repr(self.ignoretext) ]
-
-        if self.encoding != 'utf_8':
-            params.append('encoding=' + repr(self.encoding))
-        if self.useblocks:
-            params.append('useblocks=True')
-        if self.prefix:
-            params.append('prefix=' + repr(self.prefix))
-        if self.suffix:
-            params.append('suffix=' + repr(self.suffix))
-
-        fileobj.write('ImportFile(%s)\n' % (', '.join(params)))
-
-    def reloadLinks(self, document):
-        '''Reload datasets linked to this file.
-
-        Returns a tuple of
-        - List of datasets read
-        - Dict of tuples containing dataset names and number of errors
-        '''
-
-        # a bit clumsy, but we need to load this into a separate document
-        # to make sure we do not overwrited non-linked data (which may
-        # be specified in the descriptor)
-        
-        tempdoc = document.__class__()
-        sr = simpleread.SimpleRead(self.descriptor)
-        
-        stream = simpleread.FileStream(
-            utils.openEncoding(self.filename, self.encoding))
-
-        sr.readData(stream,
-                    useblocks=self.useblocks,
-                    ignoretext=self.ignoretext)
-        sr.setInDocument(tempdoc, linkedfile=self,
-                         prefix=self.prefix, suffix=self.suffix)
-
-        errors = sr.getInvalidConversions()
-
-        self._deleteLinkedDatasets(document)
-        read = self._moveReadDatasets(tempdoc, document)
-
-        # returns list of datasets read, and a dict of variables with number
-        # of errors
-        return (read, errors)
-
-class Linked2DFile(LinkedFileBase):
-    '''Class representing a file linked to a 2d dataset.'''
-
-    def __init__(self, filename, datasets):
-        self.filename = filename
-        self.datasets = datasets
-
-        self.xrange = None
-        self.yrange = None
-        self.invertrows = None
-        self.invertcols = None
-        self.transpose = None
-        self.prefix = ''
-        self.suffix = ''
-        self.encoding = 'utf_8'
-
-    def saveToFile(self, fileobj, relpath=None):
-        '''Save the link to the document file.'''
-
-        args = [repr(self._getSaveFilename(relpath)), repr(self.datasets)]
-        for p in ('xrange', 'yrange', 'invertrows', 'invertcols', 'transpose',
-                  'prefix', 'suffix', 'encoding'):
-            v = getattr(self, p)
-            if (v is not None) and (v != ''):
-                args.append( '%s=%s' % (p, repr(v)) )
-        args.append('linked=True')
-
-        fileobj.write('ImportFile2D(%s)\n' % ', '.join(args))
-
-    def reloadLinks(self, document):
-        """Reload datasets linked to this file."""
-
-        op = operations.OperationDataImport2D(
-            self.datasets, filename=self.filename,
-            xrange=self.xrange, yrange=self.yrange,
-            transpose=self.transpose,
-            invertrows=self.invertrows, invertcols=self.invertcols,
-            prefix=self.prefix, suffix=self.suffix, encoding=self.encoding)
-        return self._reloadViaOperation(document, op)
-
-class LinkedFITSFile(LinkedFileBase):
-    """Links a FITS file to the data."""
-    
-    def __init__(self, dsname, filename, hdu, columns):
-        '''Initialise the linked file object
-
-        dsname is name of dataset
-        filename is filename to load data from
-        hdu is the hdu to load the data from
-        columns is a list of columns for data, sym, pos and neg data
-        '''
-        
-        self.dsname = dsname
-        self.filename = filename
-        self.hdu = hdu
-        self.columns = columns
-
-    def saveToFile(self, fileobj, relpath=None):
-        '''Save the link to the document file.'''
-
-        args = [self.dsname, self._getSaveFilename(relpath), self.hdu]
-        args = [repr(i) for i in args]
-        for c, a in izip(self.columns,
-                         ('datacol', 'symerrcol',
-                          'poserrcol', 'negerrcol')):
-            if c is not None:
-                args.append('%s=%s' % (a, repr(c)))
-        args.append('linked=True')
-
-        fileobj.write('ImportFITSFile(%s)\n' % ', '.join(args))
-
-    def reloadLinks(self, document):
-        '''Reload datasets linked to this file.'''
-
-        op = operations.OperationDataImportFITS(
-            self.dsname, self.filename, self.hdu,
-            datacol = self.columns[0], symerrcol = self.columns[1],
-            poserrcol = self.columns[2], negerrcol = self.columns[3],
-            linked=True )
-
-        # don't use applyoperation interface as we don't want to be undoable
-        op.do(document)
-        
-        return ([self.dsname], {self.dsname: 0})
-
-class LinkedCSVFile(LinkedFileBase):
-    """A CSV file linked to datasets."""
-
-    def __init__(self, params):
-        """Read CSV data from filename
-        params is a ParamsCSV object
-        """
-        self.filename = params.filename
-        self.params = params
-
-    def saveToFile(self, fileobj, relpath=None):
-        """Save the link to the document file."""
-
-        paramsout = [repr(self._getSaveFilename(relpath)),
-                     'linked=True']
-
-        # add parameters which aren't defaults
-        for param, default in sorted(self.params.defaults.items()):
-            v = getattr(self.params, param)
-            if v != default:
-                paramsout.append('%s=%s' % (param, repr(v)))
-
-        fileobj.write('ImportFileCSV(%s)\n' % (', '.join(paramsout)))
-
-    def reloadLinks(self, document):
-        """Reload any linked data from the CSV file."""
-
-        # again, this is messy as we have to make sure we don't
-        # overwrite any non-linked data
-
-        op = operations.OperationDataImportCSV(self.params)
-        return self._reloadViaOperation(document, op)
-
-class LinkedFilePlugin(LinkedFileBase):
-    """Represent a file linked using an import plugin."""
-
-    def __init__(self, plugin, filename, pluginparams, encoding='utf_8',
-                 prefix='', suffix=''):
-        """Setup the link with the plugin-read file."""
-        self.plugin = plugin
-        self.filename = filename
-        self.encoding = encoding
-        self.prefix = prefix
-        self.suffix = suffix
-        self.pluginparams = pluginparams
-
-    def saveToFile(self, fileobj, relpath=None):
-        """Save the link to the vsz document file."""
-
-        params = [repr(self.plugin),
-                  repr(self._getSaveFilename(relpath)),
-                  'linked=True']
-        if self.encoding != 'utf_8':
-            params.append('encoding=' + repr(self.encoding))
-        if self.prefix:
-            params.append('prefix=' + repr(self.prefix))
-        if self.suffix:
-            params.append('suffix=' + repr(self.suffix))
-        for name, val in self.pluginparams.iteritems():
-            params.append('%s=%s' % (name, repr(val)))
-
-        fileobj.write('ImportFilePlugin(%s)\n' % (', '.join(params)))
-
-    def reloadLinks(self, document):
-        """Reload data from file."""
-
-        op = operations.OperationDataImportPlugin(
-            self.plugin, self.filename, 
-            encoding=self.encoding, prefix=self.prefix, suffix=self.suffix,
-            **self.pluginparams)
-        return self._reloadViaOperation(document, op)
-        
 class DatasetException(Exception):
     """Raised with dataset errors."""
     pass
@@ -461,6 +156,9 @@ class DatasetBase(object):
         # file this dataset is linked to
         self.linked = linked
 
+        # tags applied to dataset
+        self.tags = set()
+
     def saveLinksToSavedDoc(self, fileobj, savedlinks, relpath=None):
         '''Save the link to the saved document, if this dataset is linked.
 
