@@ -64,7 +64,7 @@ def getSuitableParent(widgettype, initialwidget):
     while parent is not None and not wc.willAllowParent(parent):
         parent = parent.parent
     return parent
-            
+
 class Document( qt4.QObject ):
     """Document class for holding the graph data.
 
@@ -89,6 +89,9 @@ class Document( qt4.QObject ):
         self.datachangeset = 0        # increased whan any dataset changes
         self.datachangesets = dict()  # each ds has an associated change set
 
+        # map tags to dataset names
+        self.datasettags = defaultdict(list)
+
         # if set, do not notify listeners of updates
         # wait under enableUpdates
         self.suspendupdates = []
@@ -428,6 +431,13 @@ class Document( qt4.QObject ):
         """Return widget for page."""
         return self.basewidget.children[pagenumber]
 
+    def datasetTags(self):
+        """Get list of all tags in datasets."""
+        tags = set()
+        for dataset in self.data.itervalues():
+            tags.update(dataset.tags)
+        return list(sorted(tags))
+
     def _writeFileHeader(self, fileobj, type):
         """Write a header to a saved file of type."""
 
@@ -442,6 +452,20 @@ class Document( qt4.QObject ):
             fileobj.write('AddCustom(%s, %s, %s)\n' %
                           tuple([repr(x) for x in vals]))
 
+    def saveDatasetTags(self, fileobj):
+        """Write dataset tags to output file"""
+
+        # get a list of all tags and which datasets have them
+        bytag = defaultdict(list)
+        for name, dataset in sorted(self.data.iteritems()):
+            for t in dataset.tags:
+                bytag[t].append(name)
+
+        # write out tags
+        for tag, val in sorted(bytag.iteritems()):
+            fileobj.write('TagDatasets(%s, %s)\n' %
+                          (repr(tag), repr(val)))
+
     def saveCustomFile(self, fileobj):
         """Export the custom settings to a file."""
 
@@ -473,6 +497,9 @@ class Document( qt4.QObject ):
         for name, dataset in sorted(self.data.items()):
             dataset.saveToFile(fileobj, name)
 
+        # save tags of datasets
+        self.saveDatasetTags(fileobj)
+
         # save the actual tree structure
         fileobj.write(self.basewidget.getSaveText())
         
