@@ -18,8 +18,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ##############################################################################
 
-# $Id: embed.py 1331 2010-07-15 20:08:41Z jeremysanders $
-
 """This module allows veusz to be embedded within other Python programs.
 For example:
 
@@ -49,9 +47,9 @@ import struct
 import new
 import cPickle
 import socket
-import random
 import subprocess
 import time
+import uuid
 
 # check remote process has this API version
 API_VERSION = 1
@@ -196,14 +194,14 @@ class Embedded(object):
 
             # here is the list of commands to try
             possiblecommands = [
-                [findpython, os.path.join(thisdir, 'embed_remote.py')],
+                [findpython, os.path.join(thisdir, 'veusz_main.py')],
                 [findexe] ]
 
         else:
             # try embed_remote.py in this directory, veusz in this directory
             # or veusz on the path in order
             possiblecommands = [ [sys.executable,
-                                  os.path.join(thisdir, 'embed_remote.py')],
+                                  os.path.join(thisdir, 'veusz_main.py')],
                                  [os.path.join(thisdir, 'veusz')],
                                  [findOnPath('veusz')] ]
 
@@ -251,17 +249,15 @@ class Embedded(object):
         if waitaccept:
             cls.serv_socket, address = cls.serv_socket.accept()
 
-        # send a secret to the remote program by secure route and
-        # check it comes back
-        # this is to check that no program has secretly connected
-        # on our port, which isn't really useful for AF_UNIX sockets
-        secret = ''.join([random.choice('ABCDEFGHUJKLMNOPQRSTUVWXYZ'
-                                        'abcdefghijklmnopqrstuvwxyz'
-                                        '0123456789')
-                          for i in xrange(16)]) + '\n'
+        # Send a secret to the remote program by secure route and
+        # check it comes back.  This is to check that no program has
+        # secretly connected on our port, which isn't really useful
+        # for AF_UNIX sockets.
+        secret = str(uuid.uuid4()) + '\n'
         stdin.write(secret)
         secretback = cls.readLenFromSocket(cls.serv_socket, len(secret))
-        assert secret == secretback
+        if secret != secretback:
+            raise RuntimeError, "Security between client and server broken"
 
         # packet length for command bytes
         cls.cmdlen = struct.calcsize('<I')
