@@ -1361,12 +1361,14 @@ build_cntr_list_p(long *np, double *xp, double *yp, int nparts, long ntotal)
     int i, j, k;
 
     all_contours = PyList_New(nparts);
+    if (all_contours == NULL) return NULL;
 
     for (i = 0; i < nparts; i++)
     {
         start = end;
         end += np[i];
         contourList = PyList_New(np[i]);
+	if (contourList == NULL) goto error;
         for (k = 0, j = start; j < end; j++, k++)
         {
             point = Py_BuildValue("(dd)", xp[j], yp[j]);
@@ -1430,6 +1432,7 @@ build_cntr_list_v2(long *np, double *xp, double *yp, int nparts, long ntotal)
     long j, k;
 
     all_contours = PyList_New(nparts);
+    if (all_contours == NULL) return NULL;
 
     k = 0;
     for (i = 0; i < nparts; i++)
@@ -1507,7 +1510,11 @@ cntr_trace(Csite *site, double levels[], int nlevels, int points, long nchunk)
     xp0 = (double *) PyMem_Malloc(ntotal * sizeof(double));
     yp0 = (double *) PyMem_Malloc(ntotal * sizeof(double));
     nseg0 = (long *) PyMem_Malloc(nparts * sizeof(long));
-    if (xp0 == NULL || yp0 == NULL || nseg0 == NULL) goto error;
+    if (xp0 == NULL || yp0 == NULL || nseg0 == NULL)
+    {
+	PyErr_NoMemory();
+	goto error;
+    }
 
     /* second pass */
     site->xcp = xp0;
