@@ -16,9 +16,18 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ##############################################################################
 
-# $Id: __init__.py 1334 2010-07-19 20:25:08Z jeremysanders $
-
 from field import *
 from datasetplugin import *
 from importplugin import *
 from toolsplugin import *
+
+# backward compatibility
+ImportDataset1D = Dataset1D
+ImportDataset2D = Dataset2D
+ImportDatasetText = DatasetText
+ImportField = Field
+ImportFieldCheck = FieldBool
+ImportFieldText = FieldText
+ImportFieldFloat = FieldFloat
+ImportFieldInt = FieldInt
+ImportFieldCombo = FieldCombo
