@@ -25,6 +25,10 @@ from itertools import izip
 import field
 
 import veusz.utils as utils
+try:
+    import veusz.helpers.qtloops as qtloops
+except ImportError:
+    pass
 
 # add an instance of your class to this list to be registered
 datasetpluginregistry = []
@@ -497,7 +501,7 @@ def combineMultipliedErrors(inds, length, data):
 class MultiplyDatasetPlugin(_OneOutputDatasetPlugin):
     """Dataset plugin to scale a dataset."""
 
-    menu = ('Compute', 'Multiply by constant',)
+    menu = ('Multiply', 'By constant',)
     name = 'Multiply'
     description_short = 'Multiply dataset by a constant'
     description_full = ('Multiply a dataset by a factor. '
@@ -528,7 +532,7 @@ class MultiplyDatasetPlugin(_OneOutputDatasetPlugin):
 class AddDatasetPlugin(_OneOutputDatasetPlugin):
     """Dataset plugin to add a constant to a dataset."""
 
-    menu = ('Compute', 'Add constant',)
+    menu = ('Add', 'Constant',)
     name = 'Add'
     description_short = 'Add a constant to a dataset'
     description_full = ('Add a dataset by adding a value. '
@@ -858,7 +862,7 @@ class MeanDatasetPlugin(_OneOutputDatasetPlugin):
 class AddDatasetsPlugin(_OneOutputDatasetPlugin):
     """Dataset plugin to mean datasets together."""
 
-    menu = ('Compute', 'Add datasets',)
+    menu = ('Add', 'Datasets',)
     name = 'Add Datasets'
     description_short = 'Add two or more datasets together'
     description_full = ('Add datasets together to make a single dataset. '
@@ -897,7 +901,7 @@ class AddDatasetsPlugin(_OneOutputDatasetPlugin):
 class SubtractDatasetPlugin(_OneOutputDatasetPlugin):
     """Dataset plugin to subtract two datasets."""
 
-    menu = ('Compute', 'Subtract datasets',)
+    menu = ('Subtract', 'Datasets',)
     name = 'Subtract Datasets'
     description_short = 'Subtract two datasets'
     description_full = ('Subtract two datasets. '
@@ -947,10 +951,69 @@ class SubtractDatasetPlugin(_OneOutputDatasetPlugin):
 
         self.dsout.update(data=data, serr=serr, perr=perr, nerr=nerr)
 
+class SubtractMeanDatasetPlugin(_OneOutputDatasetPlugin):
+    """Dataset plugin to subtract mean from dataset."""
+
+    menu = ('Subtract', 'Mean',)
+    name = 'Subtract Mean'
+    description_short = 'Subtract mean from dataset'
+    description_full = ('Subtract mean from dataset,'
+                        ' optionally dividing by standard deviation.')
+
+    def __init__(self):
+        """Define fields."""
+        self.fields = [
+            field.FieldDataset('ds_in', 'Input dataset 1'),
+            field.FieldBool('divstddev', 'Divide by standard deviation'),
+            field.FieldDataset('ds_out', 'Output dataset name'),
+            ]
+
+    def updateDatasets(self, fields, helper):
+        """Do scaling of dataset."""
+
+        dsin = helper.getDataset(fields['ds_in'])
+
+        vals = dsin.data
+        mean = vals[N.isfinite(vals)].mean()
+        vals = vals - mean
+
+        if fields['divstddev']:
+            vals /= vals[N.isfinite(vals)].std()
+
+        self.dsout.update(
+            data=vals, serr=dsin.serr, perr=dsin.perr, nerr=dsin.nerr)
+
+class SubtractMinimumDatasetPlugin(_OneOutputDatasetPlugin):
+    """Dataset plugin to subtract minimum from dataset."""
+
+    menu = ('Subtract', 'Minimum',)
+    name = 'Subtract Minimum'
+    description_short = 'Subtract minimum from dataset'
+    description_full = 'Subtract the minimum value from a dataset'
+
+    def __init__(self):
+        """Define fields."""
+        self.fields = [
+            field.FieldDataset('ds_in', 'Input dataset 1'),
+            field.FieldDataset('ds_out', 'Output dataset name'),
+            ]
+
+    def updateDatasets(self, fields, helper):
+        """Do scaling of dataset."""
+
+        dsin = helper.getDataset(fields['ds_in'])
+
+        vals = dsin.data
+        minval = vals[N.isfinite(vals)].min()
+        vals = vals - minval
+
+        self.dsout.update(
+            data=vals, serr=dsin.serr, perr=dsin.perr, nerr=dsin.nerr)
+
 class MultiplyDatasetsPlugin(_OneOutputDatasetPlugin):
     """Dataset plugin to multiply two or more datasets."""
 
-    menu = ('Compute', 'Multiply datasets',)
+    menu = ('Multiply', 'Datasets',)
     name = 'Multiply Datasets'
     description_short = 'Multiply two or more datasets'
     description_full = ('Multiply two or more datasets. '
@@ -989,7 +1052,7 @@ class MultiplyDatasetsPlugin(_OneOutputDatasetPlugin):
 class DivideDatasetsPlugin(_OneOutputDatasetPlugin):
     """Dataset plugin to divide two datasets."""
 
-    menu = ('Compute', 'Divide datasets',)
+    menu = ('Divide', 'Datasets',)
     name = 'Divide Datasets'
     description_short = ('Compute ratio or fractional difference'
                          ' between two datasets')
@@ -1110,6 +1173,47 @@ class ExtremesDatasetPlugin(DatasetPlugin):
             mean = tot / num
             self.dserror.update(data=mean, nerr=minvals-mean, perr=maxvals-mean)
 
+class CumulativePlugin(_OneOutputDatasetPlugin):
+    """Compute cumulative values."""
+
+    menu = ('Compute', 'Cumulative value',)
+    name = 'Cumulative'
+    description_short = 'Compute the cumulative value of a dataset'
+    description_full = ('Compute the cumulative value of a dataset. '
+                        ' Error bars are combined.\n'
+                        'Default behaviour is to accumulate from start.')
+
+    def __init__(self):
+        """Define fields."""
+        self.fields = [
+            field.FieldDataset('ds_in', 'Input dataset'),
+            field.FieldBool('fromend', 'Compute cumulative value from end'),
+            field.FieldDataset('ds_out', 'Output dataset'),
+            ]
+
+    def updateDatasets(self, fields, helper):
+        """Do accumulation."""
+
+        ds_in = helper.getDataset(fields['ds_in'])
+        fromend = fields['fromend']
+
+        def cumsum(v):
+            """Compute cumulative, handing nans and reverse."""
+            v = N.array(v)
+            if fromend: v = v[::-1]
+            v[ N.logical_not(N.isfinite(v)) ] = 0.
+            c = N.cumsum(v)
+            if fromend: c = c[::-1]
+            return c
+
+        # compute cumulative values
+        data, serr, perr, nerr = ds_in.data, ds_in.serr, ds_in.perr, ds_in.nerr
+        data = cumsum(data)
+        if serr is not None: serr = N.sqrt( cumsum(serr**2) )
+        if perr is not None: perr = N.sqrt( cumsum(perr**2) )
+        if nerr is not None: nerr = -N.sqrt( cumsum(nerr**2) )
+        self.dsout.update(data=data, serr=serr, perr=perr, nerr=nerr)
+
 class DemultiplexPlugin(DatasetPlugin):
     """Dataset plugin to split a dataset into multiple datasets, element-by-element."""
 
@@ -1200,7 +1304,7 @@ class PolarToCartesianPlugin(DatasetPlugin):
         self.y_out.update(data=y)
 
 class FilterDatasetPlugin(_OneOutputDatasetPlugin):
-    """Dataset plugin to add a constant to a dataset."""
+    """Dataset plugin to filter a dataset using an expression."""
 
     menu = ('Filter', 'Expression',)
     name = 'FilterExpression'
@@ -1241,15 +1345,90 @@ class FilterDatasetPlugin(_OneOutputDatasetPlugin):
 
         self.dsout.update(data=data, serr=serr, perr=perr, nerr=nerr)
 
+class MovingAveragePlugin(_OneOutputDatasetPlugin):
+    """Compute moving average for dataset."""
+
+    menu = ('Filtering', 'Moving Average',)
+    name = 'MovingAverage'
+    description_short = 'Compute moving average for regularly spaced data'
+    description_full = ('Compute moving average for regularly spaced data.'
+                        'Average is computed either\nside of each data point '
+                        'by number of points given.')
+    
+    def __init__(self):
+        """Define fields."""
+        self.fields = [
+            field.FieldDataset('ds_in', 'Input dataset'),
+            field.FieldInt('width', 'Points either side of point to average',
+                           default=1, minval=0),
+            field.FieldBool('weighterrors', 'Weight by error bars',
+                            default=True),
+            field.FieldDataset('ds_out', 'Output dataset'),
+            ]
+    
+    def updateDatasets(self, fields, helper):
+        """Do shifting of dataset."""
+        ds_in = helper.getDataset(fields['ds_in'])
+        weights = None
+        if fields['weighterrors']:
+            if ds_in.serr is not None:
+                weights = 1. / ds_in.serr**2
+            elif ds_in.perr is not None and ds_in.nerr is not None:
+                weights = 1. / ( (ds_in.perr**2+ds_in.nerr**2)/2. )
+        width = fields['width']
+        data = qtloops.rollingAverage(ds_in.data, weights, width)
+        self.dsout.update(data=data)
+
+class LinearInterpolatePlugin(_OneOutputDatasetPlugin):
+    """Do linear interpolation of data."""
+
+    menu = ('Filtering', 'Linear interpolation',)
+    name = 'LinearInterpolation'
+    description_short = 'Linear interpolation of x,y data'
+    description_full = ("Compute linear interpolation of x,y data.\n"
+                        "Given datasets for y = f(x), compute y' = f(x'), "
+                        "using linear interpolation.\n"
+                        "Assumes x dataset increases in value.")
+
+    def __init__(self):
+        """Define fields."""
+        self.fields = [
+            field.FieldDataset('ds_x', 'Input dataset x'),
+            field.FieldDataset('ds_y', 'Input dataset y'),
+            field.FieldDataset('ds_xprime', "Input dataset x'"),
+            field.FieldBool('edgenan', 'Use nan for values outside x range'),
+            field.FieldDataset('ds_out', "Output dataset y'"),
+            ]
+
+    def updateDatasets(self, fields, helper):
+        """Do shifting of dataset."""
+        ds_x = helper.getDataset(fields['ds_x']).data
+        ds_y = helper.getDataset(fields['ds_y']).data
+        ds_xprime = helper.getDataset(fields['ds_xprime']).data
+
+        minlenin = min( len(ds_x), len(ds_y) )
+        pad = None
+        if fields['edgenan']:
+            pad = N.nan
+
+        interpol = N.interp(ds_xprime,
+                            ds_x[:minlenin], ds_y[:minlenin],
+                            left=pad, right=pad)
+
+        self.dsout.update(data=interpol)
+
 datasetpluginregistry += [
     AddDatasetPlugin,
     AddDatasetsPlugin,
     SubtractDatasetPlugin,
-    MeanDatasetPlugin,
+    SubtractMeanDatasetPlugin,
+    SubtractMinimumDatasetPlugin,
     MultiplyDatasetPlugin,
     MultiplyDatasetsPlugin,
     DivideDatasetsPlugin,
+    MeanDatasetPlugin,
     ExtremesDatasetPlugin,
+    CumulativePlugin,
 
     ConcatenateDatasetPlugin,
     InterleaveDatasetPlugin,
@@ -1262,4 +1441,7 @@ datasetpluginregistry += [
     PolarToCartesianPlugin,
 
     FilterDatasetPlugin,
+
+    MovingAveragePlugin,
+    LinearInterpolatePlugin,
     ]
