@@ -117,7 +117,14 @@ class DatasetNode(TMNode):
 
         c = self.cols[column]
         if c == "name":
-            return qt4.QVariant(textwrap.fill(ds.description(), 40))
+            text = ds.description()
+            if text is None:
+                text = ''
+
+            if ds.tags:
+                text += '\n\nTags: %s' % (' '.join(list(sorted(ds.tags))))
+
+            return qt4.QVariant(textwrap.fill(text, 40))
         elif c == "size" or (c == 'type' and 'size' not in self.cols):
             text = ds.userPreview()
             # add preview of dataset if possible
@@ -201,18 +208,26 @@ class DatasetRelationModel(TreeModel):
         """Should dataset be filtered out by filter options."""
         filterout = False
 
-        # is filter text not in node text
-        if ( self.filter != "" and
-             all([d.find(self.filter)<0 for d in node.data]) ):
-            filterout = True
+        # is filter text not in node text or text
+        keep = True
+        if self.filter != "":
+            keep = False
+            if any([t.find(self.filter) >= 0 for t in ds.tags]):
+                keep = True
+            if any([t.find(self.filter) >= 0 for t in node.data]):
+                keep = True
+        # check dimensions haven't been filtered
         if ( self.filterdims is not None and
              ds.dimensions not in self.filterdims ):
             filterout = True
+        # check type hasn't been filtered
         if ( self.filterdtype is not None and
              ds.datatype not in self.filterdtype ):
             filterout = True
 
-        return filterout
+        if filterout:
+            return True
+        return not keep
 
     def makeGrpTreeNone(self):
         """Make tree with no grouping."""
@@ -241,11 +256,12 @@ class DatasetRelationModel(TreeModel):
             # check whether filtered out
             if not self.datasetFilterOut(ds, child):
                 # get group
-                grp = grouper(ds)
-                if grp not in grpnodes:
-                    grpnodes[grp] = GrpNodeClass( (grp,), None )
-                # add to group
-                grpnodes[grp].insertChildSorted(child)
+                grps = grouper(ds)
+                for grp in grps:
+                    if grp not in grpnodes:
+                        grpnodes[grp] = GrpNodeClass( (grp,), None )
+                    # add to group
+                    grpnodes[grp].insertChildSorted(child)
 
         return treeFromList(grpnodes.values(), coltitles)
 
@@ -254,7 +270,7 @@ class DatasetRelationModel(TreeModel):
         return self.makeGrpTree(
             ("Dataset", "Size", "Type"),
             ("name", "size", "type"),
-            lambda ds: datasetLinkFile(ds),
+            lambda ds: (datasetLinkFile(ds),),
             FilenameNode
             )
 
@@ -263,7 +279,7 @@ class DatasetRelationModel(TreeModel):
         return self.makeGrpTree(
             ("Dataset", "Type", "Filename"),
             ("name", "type", "linkfile"),
-            lambda ds: ds.userSize(),
+            lambda ds: (ds.userSize(),),
             TMNode
             )
 
@@ -272,7 +288,23 @@ class DatasetRelationModel(TreeModel):
         return self.makeGrpTree(
             ("Dataset", "Size", "Filename"),
             ("name", "size", "linkfile"),
-            lambda ds: ds.dstype,
+            lambda ds: (ds.dstype,),
+            TMNode
+            )
+
+    def makeGrpTreeTags(self):
+        """Make a tree of datasets grouped by tags."""
+
+        def getgrp(ds):
+            if ds.tags:
+                return list(sorted(ds.tags))
+            else:
+                return [u"None"]
+
+        return self.makeGrpTree(
+            ("Dataset", "Size", "Type", "Filename"),
+            ("name", "size", "type", "linkfile"),
+            getgrp,
             TMNode
             )
 
@@ -308,6 +340,7 @@ class DatasetRelationModel(TreeModel):
             "filename": self.makeGrpTreeFilename,
             "size": self.makeGrpTreeSize,
             "type": self.makeGrpTreeType,
+            "tags": self.makeGrpTreeTags,
             }[self.grouping]()
 
         self.syncTree(tree)
@@ -334,6 +367,7 @@ class DatasetsNavigatorTree(qt4.QTreeView):
                                           filterdtype=filterdtype)
 
         self.setModel(self.model)
+        self.setSelectionMode(qt4.QTreeView.ExtendedSelection)
         self.setSelectionBehavior(qt4.QTreeView.SelectRows)
         self.setUniformRowHeights(True)
         self.setContextMenuPolicy(qt4.Qt.CustomContextMenu)
@@ -394,20 +428,34 @@ class DatasetsNavigatorTree(qt4.QTreeView):
 
     def showContextMenu(self, pt):
         """Context menu for nodes."""
-        idx = self.currentIndex()
-        if idx.isValid():
-            node = self.model.objFromIndex(idx)
-        else:
-            node = None
+
+        # get selected nodes
+        idxs = self.selectionModel().selection().indexes()
+        nodes = [ self.model.objFromIndex(i)
+                  for i in idxs if i.column() == 0 ]
+
+        # unique list of types of nodes
+        types = utils.unique([ type(n) for n in nodes ])
 
         menu = qt4.QMenu()
-        if isinstance(node, DatasetNode):
-            self.datasetContextMenu(node, menu)
-        elif isinstance(node, FilenameNode):
-            self.filenameContextMenu(node, menu)
+        # put contexts onto submenus if multiple types selected
+        if DatasetNode in types:
+            thismenu = menu
+            if len(types) > 1:
+                thismenu = menu.addMenu("Datasets")
+            self.datasetContextMenu(
+                [n for n in nodes if isinstance(n, DatasetNode)],
+                thismenu)
+        elif FilenameNode in types:
+            thismenu = menu
+            if len(types) > 1:
+                thismenu = menu.addMenu("Files")
+            self.filenameContextMenu(
+                [n for n in nodes if isinstance(n, FilenameNode)],
+                thismenu)
 
         def _paste():
-            """Paste dataset."""
+            """Paste dataset(s)."""
             if document.isDataMime():
                 mime = qt4.QApplication.clipboard().mimeData()
                 self.doc.applyOperation(document.OperationDataPaste(mime))
@@ -419,72 +467,129 @@ class DatasetsNavigatorTree(qt4.QTreeView):
         if len( menu.actions() ) != 0:
             menu.exec_(self.mapToGlobal(pt))
 
-    def datasetContextMenu(self, dsnode, menu):
+    def datasetContextMenu(self, dsnodes, menu):
         """Return context menu for datasets."""
         import veusz.dialogs.dataeditdialog as dataeditdialog
-        dataset = dsnode.dataset()
-        dsname = dsnode.datasetName()
+
+        datasets = [d.dataset() for d in dsnodes]
+        dsnames = [d.datasetName() for d in dsnodes]
 
         def _edit():
             """Open up dialog box to recreate dataset."""
-            dataeditdialog.recreate_register[type(dataset)](
-                self.mainwindow, self.doc, dataset, dsname)
+            for dataset, dsname in zip(datasets, dsnames):
+                if type(dataset) in dataeditdialog.recreate_register:
+                    dataeditdialog.recreate_register[type(dataset)](
+                        self.mainwindow, self.doc, dataset, dsname)
         def _edit_data():
             """Open up data edit dialog."""
-            dialog = self.mainwindow.slotDataEdit(editdataset=dsname)
+            for dataset, dsname in zip(datasets, dsnames):
+                if type(dataset) not in dataeditdialog.recreate_register:
+                    dialog = self.mainwindow.slotDataEdit(editdataset=dsname)
         def _delete():
             """Simply delete dataset."""
-            self.doc.applyOperation(document.OperationDatasetDelete(dsname))
+            self.doc.applyOperation(
+                document.OperationMultiple(
+                    [document.OperationDatasetDelete(n) for n in dsnames],
+                    descr='delete dataset(s)'))
         def _unlink_file():
             """Unlink dataset from file."""
-            self.doc.applyOperation(document.OperationDatasetUnlinkFile(dsname))
+            self.doc.applyOperation(
+                document.OperationMultiple(
+                    [document.OperationDatasetUnlinkFile(n)
+                     for d,n in zip(datasets,dsnames)
+                     if d.canUnlink() and d.linked],
+                    descr='unlink dataset(s)'))
         def _unlink_relation():
             """Unlink dataset from relation."""
-            self.doc.applyOperation(document.OperationDatasetUnlinkRelation(dsname))
+            self.doc.applyOperation(
+                document.OperationMultiple(
+                    [document.OperationDatasetUnlinkRelation(n)
+                     for d,n in zip(datasets,dsnames)
+                     if d.canUnlink() and not d.linked],
+                    descr='unlink dataset(s)'))
         def _copy():
             """Copy data to clipboard."""
-            mime = document.generateDatasetsMime([dsname], self.doc)
+            mime = document.generateDatasetsMime(dsnames, self.doc)
             qt4.QApplication.clipboard().setMimeData(mime)
 
-        if type(dataset) in dataeditdialog.recreate_register:
+        # editing
+        recreate = [type(d) in dataeditdialog.recreate_register
+                    for d in datasets]
+        if any(recreate):
             menu.addAction("Edit", _edit)
-        else:
+        if not all(recreate):
             menu.addAction("Edit data", _edit_data)
 
+        # deletion
         menu.addAction("Delete", _delete)
-        if dataset.canUnlink():
-            if dataset.linked:
-                menu.addAction("Unlink file", _unlink_file)
-            else:
-                menu.addAction("Unlink relation", _unlink_relation)
 
+        # linking
+        unlink_file = [d.canUnlink() and d.linked for d in datasets]
+        if any(unlink_file):
+            menu.addAction("Unlink file", _unlink_file)
+        unlink_relation = [d.canUnlink() and not d.linked for d in datasets]
+        if any(unlink_relation):
+            menu.addAction("Unlink relation", _unlink_relation)
+
+        # tagging submenu
+        tagmenu = menu.addMenu("Tags")
+        for tag in self.doc.datasetTags():
+            def toggle(tag=tag):
+                state = [tag in d.tags for d in datasets]
+                if all(state):
+                    op = document.OperationDataUntag
+                else:
+                    op = document.OperationDataTag
+                self.doc.applyOperation(op(tag, dsnames))
+
+            a = tagmenu.addAction(tag, toggle)
+            a.setCheckable(True)
+            state = [tag in d.tags for d in datasets]
+
+            a.setChecked(all([tag in d.tags for d in datasets]))
+
+        def addtag():
+            tag, ok = qt4.QInputDialog.getText(
+                self, "New tag", "Enter new tag")
+            if ok:
+                tag = unicode(tag).strip().replace(' ', '')
+                if tag:
+                    self.doc.applyOperation( document.OperationDataTag(
+                            tag, dsnames) )
+        tagmenu.addAction("Add...", addtag)
+
+        # copy
         menu.addAction("Copy", _copy)
 
-        useasmenu = menu.addMenu("Use as")
-        if dataset is not None:
-            self.getMenuUseAs(useasmenu, dataset)
+        if len(datasets) == 1:
+            useasmenu = menu.addMenu("Use as")
+            self.getMenuUseAs(useasmenu, datasets[0])
 
-    def filenameContextMenu(self, node, menu):
+    def filenameContextMenu(self, nodes, menu):
         """Return context menu for filenames."""
 
         from veusz.dialogs.reloaddata import ReloadData
-        filename = node.filename()
-        if filename == '/':
-            # non linked filename node
-            return None
+
+        filenames = [n.filename() for n in nodes if n.filename() != '/']
+        if not filenames:
+            return
 
         def _reload():
             """Reload data in this file."""
-            d = ReloadData(self.doc, self.mainwindow, filenames=set([filename]))
+            d = ReloadData(self.doc, self.mainwindow, filenames=set(filenames))
             self.mainwindow.showDialog(d)
         def _unlink_all():
             """Unlink all datasets associated with file."""
             self.doc.applyOperation(
-                document.OperationDatasetUnlinkByFile(filename))
+                document.OperationMultiple(
+                    [document.OperationDatasetUnlinkByFile(f) for f in filenames],
+                    descr='unlink by file'))
         def _delete_all():
             """Delete all datasets associated with file."""
             self.doc.applyOperation(
-                document.OperationDatasetDeleteByFile(filename))
+                document.OperationMultiple(
+                    [document.OperationDatasetDeleteByFile(f) for f in filenames],
+                    descr='delete by file'))
 
         menu.addAction("Reload", _reload)
         menu.addAction("Unlink all", _unlink_all)
@@ -545,12 +650,13 @@ class DatasetBrowser(qt4.QWidget):
     """Widget which shows the document's datasets."""
 
     # how datasets can be grouped
-    grpnames = ("none", "filename", "type", "size")
+    grpnames = ("none", "filename", "type", "size", "tags")
     grpentries = {
         "none": "None",
         "filename": "Filename",
         "type": "Type", 
-        "size": "Size"
+        "size": "Size",
+        "tags": "Tags",
         }
 
     def __init__(self, thedocument, mainwin, parent, readonly=False,
