@@ -27,6 +27,7 @@ s.fromText('42')
 """
 
 import re
+import sys
 
 import numpy as N
 import veusz.qtall as qt4
@@ -358,10 +359,13 @@ class SettingBackwardCompat(Setting):
 
     typename = 'backward-compat'
 
-    def __init__(self, name, newrelpath, val, **args):
+    def __init__(self, name, newrelpath, val, translatefn = None,
+                 **args):
         """Point this setting to another.
         newrelpath is a path relative to this setting's parent
         """
+
+        self.translatefn = translatefn
         Setting.__init__(self, name, val, **args)
         self.relpath = newrelpath.split('/')
 
@@ -381,6 +385,8 @@ class SettingBackwardCompat(Setting):
 
     def set(self, val):
         if self.parent is not None and not isinstance(val, Reference):
+            if self.translatefn:
+                val = self.translatefn(val)
             self.getForward().set(val)
 
     def isDefault(self):
@@ -390,7 +396,8 @@ class SettingBackwardCompat(Setting):
         return self.getForward().get()
 
     def copy(self):
-        return self._copyHelper(('/'.join(self.relpath),), (), {})
+        return self._copyHelper(('/'.join(self.relpath),), (),
+                                {'translatefn': self.translatefn})
 
     def makeControl(self, *args):
         return None
@@ -833,10 +840,18 @@ class Choice(Setting):
     typename = 'choice'
 
     def __init__(self, name, vallist, val, **args):
-        """Setting val must be in vallist."""
-        
+        """Setting val must be in vallist.
+        descriptions is an optional addon to put a tooltip on each item
+        in the control.
+        """
+
         assert type(vallist) in (list, tuple)
+
         self.vallist = vallist
+        self.descriptions = args.get('descriptions', None)
+        if self.descriptions:
+            del args['descriptions']
+
         Setting.__init__(self, name, val, **args)
 
     def copy(self):
@@ -857,9 +872,10 @@ class Choice(Setting):
             return text
         else:
             raise InvalidType
-        
+
     def makeControl(self, *args):
-        return controls.Choice(self, False, self.vallist, *args)
+        argsv = {'descriptions': self.descriptions}
+        return controls.Choice(self, False, self.vallist, *args, **argsv)
 
 class ChoiceOrMore(Setting):
     """One out of a list of strings, or anything else."""
@@ -1620,7 +1636,12 @@ class FillSet(Setting):
         [('solid', 'color', False), ...]
 
         These are color, fill style, and hide or
-        color, fill style, transparency and hide
+        color, fill style, and hide
+
+        (style, color, hide,
+        [optional transparency, linewidth,
+         linestyle, spacing, backcolor, backtrans, backhide]])
+
         """
 
         if type(val) not in (list, tuple):
@@ -1629,13 +1650,14 @@ class FillSet(Setting):
         # check each entry in the list is appropriate
         for fill in val:
             try:
-                style, color, hide = fill
+                style, color, hide = fill[:3]
             except ValueError:
                 raise InvalidType
 
             if ( not isinstance(color, basestring) or
-                 style not in FillStyle._fillstyles or
-                 type(hide) not in (int, bool) ):
+                 style not in utils.extfillstyles or
+                 type(hide) not in (int, bool) or
+                 len(fill) not in (3, 10) ):
                 raise InvalidType
 
         return val
@@ -1643,32 +1665,31 @@ class FillSet(Setting):
     def makeControl(self, *args):
         """Make specialised lineset control."""
         return controls.FillSet(self, *args)
-    
-    def makeBrush(self, row):
-        """Make a Qt brush corresponding to the row given.
 
-        If row is outside of range, then cycle
-        """
+    def returnBrushExtended(self, row):
+        """Return BrushExtended for the row."""
+        import collections
+        s = collections.BrushExtended('tempbrush')
 
         if len(self.val) == 0:
-            return qt4.QBrush()
+            s.hide = True
         else:
-            row = row % len(self.val)
-            style, color, hide = self.val[row]
-            col = utils.extendedColorToQColor(color)
-            b = qt4.QBrush(col, FillStyle._fillcnvt[style])
-            if hide:
-                b.setStyle(qt4.Qt.NoBrush)
-            return b
-    
-class ImageFilename(Str):
-    """Represents an image filename setting."""
+            v = self.val[row % len(self.val)]
+            s.style = v[0]
+            col = utils.extendedColorToQColor(v[1])
+            if col.alpha() != 255:
+                s.transparency = int(100 - col.alphaF()*100)
+                col.setAlpha(255)
+                s.color = col.name()
+            else:
+                s.color = v[1]
+            s.hide = v[2]
+            if len(v) == 10:
+                (s.transparency, s.linewidth, s.linestyle,
+                 s.patternspacing, s.backcolor,
+                 s.backtransparency, s.backhide) = v[3:]
+        return s
 
-    typename = 'filename-image'
-
-    def makeControl(self, *args):
-        return controls.Filename(self, 'image', *args)
-    
 class Filename(Str):
     """Represents a filename setting."""
 
@@ -1676,7 +1697,20 @@ class Filename(Str):
 
     def makeControl(self, *args):
         return controls.Filename(self, 'file', *args)
-    
+
+    def convertTo(self, val):
+        if sys.platform == 'win32':
+            val = val.replace('\\', '/')
+        return val
+
+class ImageFilename(Filename):
+    """Represents an image filename setting."""
+
+    typename = 'filename-image'
+
+    def makeControl(self, *args):
+        return controls.Filename(self, 'image', *args)
+
 class FontFamily(Str):
     """Represents a font family."""
 
@@ -1697,7 +1731,7 @@ class ErrorStyle(Choice):
         'none',
         'bar', 'barends', 'box', 'diamond', 'curve',
         'barbox', 'bardiamond', 'barcurve',
-        'boxfill',
+        'boxfill', 'diamondfill',
         'fillvert', 'fillhorz',
         'linevert', 'linehorz',
         'linevertbar', 'linehorzbar'
@@ -1783,6 +1817,55 @@ class BoolSwitch(Bool):
         return self._copyHelper((), (), {'settingsfalse': self.sfalse,
                                          'settingstrue': self.strue})
 
+class ChoiceSwitch(Choice):
+    """Show or hide other settings based on the choice given here."""
+
+    def __init__(self, name, vallist, value, settingstrue=[], settingsfalse=[],
+                 showfn=lambda val: True, **args):
+        """Enables/disables a set of settings if True or False
+        settingsfalse and settingstrue are lists of names of settings
+        which are hidden/shown to user depending on showfn(val)."""
+
+        self.sfalse = settingsfalse
+        self.strue = settingstrue
+        self.showfn = showfn
+        Choice.__init__(self, name, vallist, value, **args)
+
+    def makeControl(self, *args):
+        return controls.ChoiceSwitch(self, *args)
+
+    def copy(self):
+        return self._copyHelper((self.vallist), (),
+                                {'settingsfalse': self.sfalse,
+                                 'settingstrue': self.strue,
+                                 'showfn': self.showfn})
+
+class FillStyleExtended(ChoiceSwitch):
+    """A setting for the different fill styles provided by Qt."""
+
+    typename = 'fill-style-ext'
+
+    _strue = ( 'linewidth', 'linestyle', 'patternspacing',
+               'backcolor', 'backtransparency', 'backhide' )
+
+    @staticmethod
+    def _ishatch(val):
+        """Is this a hatching fill?"""
+        return not ( val == 'solid' or val.find('dense') >= 0 )
+
+    def __init__(self, name, value, **args):
+        ChoiceSwitch.__init__(self, name, utils.extfillstyles, value,
+                              settingstrue=self._strue, settingsfalse=(),
+                              showfn=self._ishatch,
+                              **args)
+
+    def copy(self):
+        """Make a copy of the setting."""
+        return self._copyHelper((), (), {})
+
+    def makeControl(self, *args):
+        return controls.FillStyleExtended(self, *args)
+
 class RotateInterval(Choice):
     '''Rotate a label with intervals given.'''
 
