@@ -31,6 +31,7 @@ defaultValues = {
     'export_antialias': True,
     'export_quality': 85,
     'export_background': '#ffffff00',
+    'export_SVG_text_as_text': False,
 
     # plot options
     'plot_updatepolicy': -1, # update on document changed
@@ -111,7 +112,7 @@ class _SettingDB(object):
             if default == 'LightBase':
                 base = qt4.qApp.palette().color(qt4.QPalette.Base)
                 if base.value() < 127:
-                    base = qt4.Qt.white
+                    base = qt4.QColor(qt4.Qt.white)
                 return base
 
             return qt4.QColor(default)
