@@ -30,12 +30,13 @@ from pdf import *
 from dates import *
 from formatting import *
 from colormap import *
+from extbrushfilling import *
 
 try:
     from veusz.helpers.qtloops import addNumpyToPolygonF, plotPathsToPainter, \
         plotLinesToPainter, plotClippedPolyline, polygonClip, \
-        plotClippedPolygon, plotBoxesToPainter
+        plotClippedPolygon, plotBoxesToPainter, addNumpyPolygonToPath
 except ImportError:
     from slowfuncs import addNumpyToPolygonF, plotPathsToPainter, \
         plotLinesToPainter, plotClippedPolyline, polygonClip, \
-        plotClippedPolygon, plotBoxesToPainter
+        plotClippedPolygon, plotBoxesToPainter, addNumpyPolygonToPath
