@@ -18,8 +18,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ###############################################################################
 
-# $Id: points.py 1280 2010-06-13 14:53:17Z jeremysanders $
-
 from itertools import izip
 
 import veusz.qtall as qt4
@@ -30,6 +28,8 @@ try:
 except ImportError:
     from slowfuncs import plotPathsToPainter
 
+import colormap
+
 """This is the symbol plotting part of Veusz
 
 There are actually several different ways symbols are plotted.
@@ -383,7 +383,7 @@ MarkerCodes = (
     )
 
 def plotMarkers(painter, xpos, ypos, markername, markersize, scaling=None,
-                clip=None):
+                clip=None, cmap=None, colorvals=None):
     """Funtion to plot an array of markers on a painter.
 
     painter: QPainter
@@ -392,6 +392,8 @@ def plotMarkers(painter, xpos, ypos, markername, markersize, scaling=None,
     markersize: size of marker to plot
     scaling: scale size of markers by array, or don't in None
     clip: rectangle if clipping wanted
+    cmap: colormap to use if colorvals is set
+    colorvals: color values 0-1 of each point if used
     """
 
     # minor optimization
@@ -411,21 +413,18 @@ def plotMarkers(painter, xpos, ypos, markername, markersize, scaling=None,
         # turn off brush
         painter.setBrush( qt4.QBrush() )
 
-    # split up into two loops as this is a critical path
-    if scaling is None:
-        plotPathsToPainter(painter, path, xpos, ypos, clip)
-    else:
-        # plot markers, scaling each one
-        s = painter.scale
-        t = painter.translate
-        d = painter.drawPath
-        r = painter.resetTransform
-        for x, y, sc in izip(xpos, ypos, scaling):
-            t(x, y)
-            s(sc, sc)
-            d(path)
-            r()
-    
+    # if using colored points
+    colorimg = None
+    if colorvals is not None:
+        # convert colors to rgb values via a 2D image and pass to function
+        trans = (1-painter.brush().color().alphaF())*100
+        color2d = colorvals.reshape( 1, len(colorvals) )
+        colorimg = colormap.applyColorMap(
+            cmap, 'linear', color2d, 0., 1., trans)
+
+    # this is the fast (C++) or slow (python) helper
+    plotPathsToPainter(painter, path, xpos, ypos, scaling, clip, colorimg)
+
     painter.restore()
 
 def plotMarker(painter, xpos, ypos, markername, markersize):
