@@ -16,8 +16,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ###############################################################################
 
-# $Id: safe_eval.py 1089 2009-10-25 17:59:57Z jeremysanders $
-
 """
 'Safe' python code evaluation
 
@@ -329,7 +327,9 @@ def checkCode(code, securityonly=False):
     if securityonly is set, then don't return errors from Python
     exceptions.
     """
-    
+
+    # compiler can't parse strings with unicode
+    code = code.encode('utf8')
     try:
         ast = compiler.parse(code)
     except SyntaxError, e:
