@@ -35,23 +35,43 @@ class IgnoreException(Exception):
     """A special exception class to be ignored by the exception handler."""
 
 def _getVeuszDirectory():
-    """Get installed directory to find files relative to this one."""
+    """Get resource and examples directories for Veusz."""
 
     if hasattr(sys, 'frozen'):
         # for pyinstaller/py2app compatability
-        dirname = os.path.dirname(os.path.abspath(sys.executable))
+        resdir = os.path.dirname(os.path.abspath(sys.executable))
         if sys.platform == 'darwin':
-            # py2app
-            return os.path.join(dirname, '..', 'Resources', 'veusz')
-        else:
-            # pyinstaller
-            return dirname
+            # special case for py2app
+            resdir = os.path.join(resdir, '..', 'Resources')
     else:
         # standard installation
-        return os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
+        resdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
 
-veuszDirectory = _getVeuszDirectory()
-exampleDirectory = os.path.join(veuszDirectory, 'examples')
+    # override data directory with symlink
+    if os.path.exists( os.path.join(resdir, 'resources') ):
+        resdir = os.path.realpath( os.path.join(resdir, 'resources') )
+
+    # override with VEUSZ_RESOURCE_DIR environment variable if necessary
+    resdir = os.environ.get('VEUSZ_RESOURCE_DIR', resdir)
+
+    # now get example directory (which may be a symlink)
+    examplesdir = os.path.realpath( os.path.join(resdir, 'examples') )
+
+    return resdir, examplesdir
+
+# get resource and example directories
+veuszDirectory, exampleDirectory = _getVeuszDirectory()
+
+def getLicense():
+    """Return license text."""
+    try:
+        f = open(os.path.join(veuszDirectory, 'COPYING'), 'rU')
+        text = f.read()
+        f.close()
+    except EnvironmentError:
+        text = ('Could not open the license file.\n'
+                'See license at http://www.gnu.org/licenses/gpl-2.0.html')
+    return text
 
 id_re = re.compile('^[A-Za-z_][A-Za-z0-9_]*$')
 def validPythonIdentifier(name):
@@ -413,3 +433,18 @@ def positionFloatingPopup(popup, widget):
 
     popup.move(x, y)
     popup.setFocus()
+
+def unique(inlist):
+    """Get unique entries in list."""
+
+    inlist = list(inlist)
+    inlist.sort()
+
+    class X: pass
+    last = X()
+    out = []
+    for x in inlist:
+        if x != last:
+            out.append(x)
+            last = x
+    return out
