@@ -43,12 +43,12 @@ import veusz.utils as utils
 
 copyr='''Veusz %s
 
-Copyright (C) Jeremy Sanders 2003-2011 <jeremy@jeremysanders.net> and contributors
+Copyright (C) Jeremy Sanders 2003-2012 <jeremy@jeremysanders.net> and contributors
 Licenced under the GNU General Public Licence (version 2 or greater)
 '''
 
 splashcopyr='''<b><font color="purple">Veusz %s<br></font></b>
-Copyright (C) Jeremy Sanders 2003-2011 and contributors<br>
+Copyright (C) Jeremy Sanders 2003-2012 and contributors<br>
 Licenced under the GPL (version 2 or greater)
 '''
 
@@ -189,6 +189,8 @@ def run():
     # have too long a wait before it shows
     import veusz.setting
     import veusz.widgets
+    import veusz.utils.vzdbus as vzdbus
+    vzdbus.setup()
 
     # for people who want to run any old script
     veusz.setting.transient_settings['unsafe_mode'] = bool(
