@@ -16,8 +16,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ##############################################################################
 
-# $Id: controlgraph.py 1411 2010-09-25 08:38:27Z jeremysanders $
-
 """
 Classes for moving widgets around
 
@@ -87,14 +85,14 @@ class _EdgeLine(qt4.QGraphicsLineItem):
 ##############################################################################
 
 class ControlMarginBox(object):
-    def __init__(self, widget, posn, maxposn, painter,
+    def __init__(self, widget, posn, maxposn, painthelper,
                  ismovable = True, isresizable = True):
         """Create control box item.
 
         widget: widget this is controllng
         posn: coordinates of box [x1, y1, x2, y2]
         maxposn: coordinates of biggest possibe box
-        painter: painter to get scaling from
+        painthelper: painterhelper to get scaling from
         ismovable: box can be moved
         isresizable: box can be resized
         """
@@ -107,9 +105,9 @@ class ControlMarginBox(object):
         self.isresizable = isresizable
 
         # we need these later to convert back to original units
-        self.page_size = painter.veusz_page_size
-        self.scaling = painter.veusz_scaling
-        self.pixperpt = painter.veusz_pixperpt
+        self.pagesize = painthelper.pagesize
+        self.scaling = painthelper.scaling
+        self.dpi = painthelper.dpi
 
     def createGraphicsItem(self):
         return _GraphMarginBox(self)
@@ -118,8 +116,7 @@ class ControlMarginBox(object):
         """A helpful routine for setting widget margins after
         moving or resizing.
 
-        This is called by the widget after receiving
-        updateControlItem
+        This is called by the widget after receiving updateControlItem
         """
         s = self.widget.settings
 
@@ -129,17 +126,15 @@ class ControlMarginBox(object):
         top = self.posn[1] - self.maxposn[1]
         bottom = self.maxposn[3] - self.posn[3]
 
-        # set up fake painter containing veusz scalings
-        fakepainter = qt4.QPainter()
-        fakepainter.veusz_page_size = self.page_size
-        fakepainter.veusz_scaling = self.scaling
-        fakepainter.veusz_pixperpt = self.pixperpt
+        # set up fake painthelper containing veusz scalings
+        helper = document.PaintHelper(self.pagesize, scaling=self.scaling,
+                                      dpi=self.dpi)
 
         # convert to physical units
-        left = s.get('leftMargin').convertInverse(left, fakepainter)
-        right = s.get('rightMargin').convertInverse(right, fakepainter)
-        top = s.get('topMargin').convertInverse(top, fakepainter)
-        bottom = s.get('bottomMargin').convertInverse(bottom, fakepainter)
+        left = s.get('leftMargin').convertInverse(left, helper)
+        right = s.get('rightMargin').convertInverse(right, helper)
+        top = s.get('topMargin').convertInverse(top, helper)
+        bottom = s.get('bottomMargin').convertInverse(bottom, helper)
 
         # modify widget margins
         operations = (
@@ -151,6 +146,33 @@ class ControlMarginBox(object):
         self.widget.document.applyOperation(
             document.OperationMultiple(operations, descr='resize margins'))
 
+    def setPageSize(self):
+        """Helper for setting document/page widget size.
+
+        This is called by the widget after receiving updateControlItem
+        """
+        s = self.widget.settings
+
+        # get margins in pixels
+        width = self.posn[2] - self.posn[0]
+        height = self.posn[3] - self.posn[1]
+
+        # set up fake painter containing veusz scalings
+        helper = document.PaintHelper(self.pagesize, scaling=self.scaling,
+                                      dpi=self.dpi)
+
+        # convert to physical units
+        width = s.get('width').convertInverse(width, helper)
+        height = s.get('height').convertInverse(height, helper)
+
+        # modify widget margins
+        operations = (
+            document.OperationSettingSet(s.get('width'), width),
+            document.OperationSettingSet(s.get('height'), height),
+            )
+        self.widget.document.applyOperation(
+            document.OperationMultiple(operations, descr='change page size'))
+
 class _GraphMarginBox(qt4.QGraphicsItem):
     """A box which can be moved or resized.
 
@@ -410,10 +432,10 @@ class ControlMovableBox(ControlMarginBox):
     the real position of the widget is
     """
 
-    def __init__(self, widget, posn, painter, crosspos=None):
+    def __init__(self, widget, posn, painthelper, crosspos=None):
         ControlMarginBox.__init__(self, widget, posn,
                                   [-10000, -10000, 10000, 10000],
-                                  painter, isresizable=False)
+                                  painthelper, isresizable=False)
         self.deltacrosspos = (crosspos[0] - self.posn[0],
                               crosspos[1] - self.posn[1])
 
@@ -528,11 +550,22 @@ class ControlAxisLine(object):
                  maxposn):
         self.widget = widget
         self.direction = direction
-        self.minpos = minpos
-        self.maxpos = maxpos
-        self.axispos = axispos
+        if minpos > maxpos:
+            minpos, maxpos = maxpos, minpos
+        self.minpos = self.minzoom = self.minorig = minpos
+        self.maxpos = self.maxzoom = self.maxorig = maxpos
+        self.axisorigpos = self.axispos = axispos
         self.maxposn = maxposn
 
+    def zoomed(self):
+        """Is this a zoom?"""
+        return self.minzoom != self.minorig or self.maxzoom != self.maxorig
+
+    def moved(self):
+        """Has axis moved?"""
+        return ( self.minpos != self.minorig or self.maxpos != self.maxorig or
+                 self.axisorigpos != self.axispos )
+
     def createGraphicsItem(self):
         return _GraphAxisLine(self)
 
@@ -540,21 +573,27 @@ class _GraphAxisLine(qt4.QGraphicsItem):
 
     curs = {True: qt4.Qt.SizeVerCursor,
             False: qt4.Qt.SizeHorCursor}
+    curs_zoom = {True: qt4.Qt.SplitVCursor,
+                 False: qt4.Qt.SplitHCursor}
 
     def __init__(self, params):
         """Line is about to be shown."""
         qt4.QGraphicsItem.__init__(self)
         self.params = params
-        self.pts = [ _ShapeCorner(self),
-                     _ShapeCorner(self) ]
+        self.pts = [ _ShapeCorner(self), _ShapeCorner(self),
+                     _ShapeCorner(self), _ShapeCorner(self) ]
         self.line = _AxisGraphicsLineItem(self)
 
-        # set correct coordinates
+        # set cursors and tooltips for items
         self.horz = (params.direction == 'horizontal')
-        endcurs = self.curs[not self.horz]
-        self.pts[0].setCursor(endcurs)
-        self.pts[1].setCursor(endcurs)
+        for p in self.pts[0:2]:
+            p.setCursor(self.curs[not self.horz])
+            p.setToolTip("Move axis ends")
+        for p in self.pts[2:]:
+            p.setCursor(self.curs_zoom[not self.horz])
+            p.setToolTip("Change axis scale")
         self.line.setCursor( self.curs[self.horz] )
+        self.line.setToolTip("Move axis position")
         self.setZValue(2.)
 
         self.updatePos()
@@ -563,47 +602,58 @@ class _GraphAxisLine(qt4.QGraphicsItem):
         """Set ends of line and line positions from stored values."""
         par = self.params
         mxp = par.maxposn
+
+        def _clip(*args):
+            """Clip positions to bounds of box given coords."""
+            par.minpos = max(par.minpos, mxp[args[0]])
+            par.maxpos = min(par.maxpos, mxp[args[1]])
+            par.axispos = max(par.axispos, mxp[args[2]])
+            par.axispos = min(par.axispos, mxp[args[3]])
+
         if self.horz:
-            # clip to box bounds
-            par.minpos = max(par.minpos, mxp[0])
-            par.maxpos = min(par.maxpos, mxp[2])
-            par.axispos = max(par.axispos, mxp[1])
-            par.axispos = min(par.axispos, mxp[3])
+            _clip(0, 2, 1, 3)
 
             # set positions
-            self.line.setPos(par.minpos, par.axispos)
-            self.line.setLine(0, 0, par.maxpos-par.minpos, 0)
+            if par.zoomed():
+                self.line.setPos(par.minzoom, par.axispos)
+                self.line.setLine(0, 0, par.maxzoom-par.minzoom, 0)
+            else:
+                self.line.setPos(par.minpos, par.axispos)
+                self.line.setLine(0, 0, par.maxpos-par.minpos, 0)
             self.pts[0].setPos(par.minpos, par.axispos)
             self.pts[1].setPos(par.maxpos, par.axispos)
+            self.pts[2].setPos(par.minzoom, par.axispos-15)
+            self.pts[3].setPos(par.maxzoom, par.axispos-15)
         else:
-            # clip to box bounds
-            par.minpos = max(par.minpos, mxp[1])
-            par.maxpos = min(par.maxpos, mxp[3])
-            par.axispos = max(par.axispos, mxp[0])
-            par.axispos = min(par.axispos, mxp[2])
+            _clip(1, 3, 0, 2)
 
             # set positions
-            self.line.setPos(par.axispos, par.minpos)
-            self.line.setLine(0, 0, 0, par.maxpos-par.minpos)
+            if par.zoomed():
+                self.line.setPos(par.axispos, par.minzoom)
+                self.line.setLine(0, 0, 0, par.maxzoom-par.minzoom)
+            else:
+                self.line.setPos(par.axispos, par.minpos)
+                self.line.setLine(0, 0, 0, par.maxpos-par.minpos)
             self.pts[0].setPos(par.axispos, par.minpos)
             self.pts[1].setPos(par.axispos, par.maxpos)
+            self.pts[2].setPos(par.axispos+15, par.minzoom)
+            self.pts[3].setPos(par.axispos+15, par.maxzoom)
 
     def updateFromCorner(self, corner, event):
         """Ends of axis have moved, so update values."""
 
         par = self.params
+        pt = (corner.y(), corner.x())[self.horz]
         # which end has moved?
         if corner is self.pts[0]:
             # horizonal or vertical axis?
-            if self.horz:
-                par.minpos = corner.x()
-            else:
-                par.minpos = corner.y()
-        else:
-            if self.horz:
-                par.maxpos = corner.x()
-            else:
-                par.maxpos = corner.y()
+            par.minpos = pt
+        elif corner is self.pts[1]:
+            par.maxpos = pt
+        elif corner is self.pts[2]:
+            par.minzoom = pt
+        elif corner is self.pts[3]:
+            par.maxzoom = pt
 
         # swap round end points if min > max
         if par.minpos > par.maxpos:
