@@ -28,6 +28,7 @@ import veusz.qtall as qt4
 
 import veusz.document as document
 import veusz.utils as utils
+import veusz.utils.vzdbus as vzdbus
 import veusz.setting as setting
 import veusz.plugins as plugins
 
@@ -54,6 +55,26 @@ import veusz.dialogs.dataeditdialog as dataeditdialog
 # shortcut to this
 setdb = setting.settingdb
 
+class DBusWinInterface(vzdbus.Object):
+    """Simple DBus interface to window for triggering actions."""
+
+    interface = 'org.veusz.actions'
+
+    def __init__(self, actions, index):
+        prefix = '/Windows/%i/Actions' % index
+        vzdbus.Object.__init__(self, vzdbus.sessionbus, prefix)
+        self.actions = actions
+
+    @vzdbus.method(dbus_interface=interface, out_signature='as')
+    def GetActions(self):
+        """Get list of actions which can be activated."""
+        return list(sorted(self.actions.keys()))
+
+    @vzdbus.method(dbus_interface=interface, in_signature='s')
+    def TriggerAction(self, action):
+        """Activate action given."""
+        self.actions[unicode(action)].trigger()
+
 class MainWindow(qt4.QMainWindow):
     """ The main window class for the application."""
 
@@ -73,12 +94,7 @@ class MainWindow(qt4.QMainWindow):
             # load document
             win.openFileInWindow(filename)
         else:
-            # add page and default graph
-            win.document.makeDefaultDoc()
-
-            # load defaults if set
-            win.loadDefaultStylesheet()
-            win.loadDefaultCustomDefinitions()
+            win.setupDefaultDoc()
 
         # try to select first graph of first page
         win.treeedit.doInitialWidgetSelect()
@@ -212,6 +228,11 @@ class MainWindow(qt4.QMainWindow):
         self.connect(self.document, qt4.SIGNAL('check_allowed_imports'),
                      self.slotAllowedImportsDoc)
 
+        # add on dbus interface
+        self.dbusdocinterface = document.DBusInterface(self.document)
+        self.dbuswininterface = DBusWinInterface(
+            self.vzactions, self.dbusdocinterface.index)
+
     def updateStatusbar(self, text):
         '''Display text for a set period.'''
         self.statusBar().showMessage(text, 2000)
@@ -247,6 +268,16 @@ class MainWindow(qt4.QMainWindow):
             urls = [u for u in urls if os.path.splitext(u)[1] == '.vsz']
             return urls
 
+    def setupDefaultDoc(self):
+        """Setup default document."""
+
+        # add page and default graph
+        self.document.makeDefaultDoc()
+
+        # load defaults if set
+        self.loadDefaultStylesheet()
+        self.loadDefaultCustomDefinitions()
+
     def loadDefaultStylesheet(self):
         """Loads the default stylesheet for the new document."""
         filename = setdb['stylesheet_default']
@@ -254,10 +285,11 @@ class MainWindow(qt4.QMainWindow):
             try:
                 self.document.applyOperation(
                     document.OperationLoadStyleSheet(filename) )
-            except IOError:
-                qt4.QMessageBox.warning(self, "Veusz",
-                                        "Unable to load default "
-                                        "stylesheet '%s'" % filename)
+            except EnvironmentError, e:
+                qt4.QMessageBox.warning(
+                    self, "Error - Veusz",
+                    "Unable to load default stylesheet '%s'\n\n%s" %
+                    (filename, e.strerror))
             else:
                 # reset any modified flag
                 self.document.setModified(False)
@@ -270,10 +302,11 @@ class MainWindow(qt4.QMainWindow):
             try:
                 self.document.applyOperation(
                     document.OperationLoadCustom(filename) )
-            except IOError:
-                qt4.QMessageBox.warning(self, "Veusz",
-                                        "Unable to load default custom "
-                                        "definitons '%s'" % filename)
+            except EnvironmentError, e:
+                qt4.QMessageBox.warning(
+                    self, "Error - Veusz",
+                    "Unable to load custom definitions '%s'\n\n%s" %
+                    (filename, e.strerror))
             else:
                 # reset any modified flag
                 self.document.setModified(False)
@@ -337,7 +370,7 @@ class MainWindow(qt4.QMainWindow):
                 """Load plugin dialog"""
                 handlePlugin(self, self.document, pluginkls)
 
-            actname = menuname + '.' + '.'.join(pluginkls.name)
+            actname = menuname + '.' + '.'.join(pluginkls.menu)
             text = pluginkls.menu[-1]
             if pluginkls.has_parameters:
                 text += '...'
@@ -845,12 +878,12 @@ class MainWindow(qt4.QMainWindow):
                 ofile = open(self.filename, 'w')
                 self.document.saveToFile(ofile)
                 self.updateStatusbar("Saved to %s" % self.filename)
-            except IOError, e:
+            except EnvironmentError, e:
                 qt4.QApplication.restoreOverrideCursor()
-                qt4.QMessageBox.critical(self, "Cannot save document",
-                                         "Cannot save document as '%s'\n"
-                                         "\n%s (error %i)" %
-                                         (self.filename,  e.strerror, e.errno))
+                qt4.QMessageBox.critical(
+                    self, "Error - Veusz",
+                    "Unable to save document as '%s'\n\n%s" %
+                    (self.filename, e.strerror))
             else:
                 # restore the cursor
                 qt4.QApplication.restoreOverrideCursor()
@@ -906,10 +939,11 @@ class MainWindow(qt4.QMainWindow):
             filename = unicode( fd.selectedFiles()[0] )
             try:
                 open(filename)
-            except IOError, e:
-                qt4.QMessageBox.critical(self, "Unable to open file",
-                                         "Unable to open file '%s'\n'%s'" %
-                                         (filename, unicode(e)))
+            except EnvironmentError, e:
+                qt4.QMessageBox.critical(
+                    self, "Error - Veusz",
+                    "Unable to open '%s'\n\n%s" %
+                    (filename, e.strerror))
                 return None
             return filename
         return None
@@ -983,12 +1017,13 @@ class MainWindow(qt4.QMainWindow):
         # read script
         try:
             script = open(filename, 'rU').read()
-        except IOError, e:
+        except EnvironmentError, e:
             qt4.QApplication.restoreOverrideCursor()
-            qt4.QMessageBox.warning(self, "Cannot open document",
-                                    "Cannot open the document '%s'\n"
-                                    "\n%s (error %i)" % (filename,
-                                                         e.strerror, e.errno))
+            qt4.QMessageBox.critical(
+                self, "Error - Veusz",
+                "Cannot open document '%s'\n\n%s" %
+                (filename, e.strerror))
+            self.setupDefaultDoc()
             return
 
         # check code for any security issues
@@ -1137,7 +1172,7 @@ class MainWindow(qt4.QMainWindow):
 
         # check there is a page
         if self.document.getNumberPages() == 0:
-            qt4.QMessageBox.warning(self, "Veusz",
+            qt4.QMessageBox.warning(self, "Error - Veusz",
                                     "No pages to export")
             return
 
@@ -1202,29 +1237,40 @@ class MainWindow(qt4.QMainWindow):
             if (ext not in validextns) and (ext not in chosenextns):
                 filename += "." + chosenextns[0]
 
-            e = document.Export( self.document,
-                                 filename,
-                                 self.plot.getPageNumber(),
-                                 bitmapdpi=setdb['export_DPI'],
-                                 pdfdpi=setdb['export_DPI_PDF'],
-                                 antialias=setdb['export_antialias'],
-                                 color=setdb['export_color'],
-                                 quality=setdb['export_quality'],
-                                 backcolor=setdb['export_background'] )
-            try:
-                e.export()
-            except (IOError, RuntimeError), inst:
-                qt4.QMessageBox.critical(self, "Veusz",
-                                         "Error exporting file:\n%s" % inst)
+            export = document.Export(
+                self.document,
+                filename,
+                self.plot.getPageNumber(),
+                bitmapdpi=setdb['export_DPI'],
+                pdfdpi=setdb['export_DPI_PDF'],
+                antialias=setdb['export_antialias'],
+                color=setdb['export_color'],
+                quality=setdb['export_quality'],
+                backcolor=setdb['export_background'],
+                svgtextastext=setdb['export_SVG_text_as_text'],
+                )
 
-            # restore the cursor
-            qt4.QApplication.restoreOverrideCursor()
+            try:
+                export.export()
+            except (RuntimeError, EnvironmentError), e:
+                if isinstance(e, EnvironmentError):
+                    msg = e.strerror
+                else:
+                    msg = unicode(e)
+
+                qt4.QApplication.restoreOverrideCursor()
+                qt4.QMessageBox.critical(
+                    self, "Error - Veusz",
+                    "Error exporting to file '%s'\n\n%s" %
+                    (filename, msg))
+            else:
+                qt4.QApplication.restoreOverrideCursor()
 
     def slotFilePrint(self):
         """Print the document."""
 
         if self.document.getNumberPages() == 0:
-            qt4.QMessageBox.warning(self, "Veusz",
+            qt4.QMessageBox.warning(self, "Error - Veusz",
                                     "No pages to print")
             return
 
