@@ -219,6 +219,9 @@ class PlotWindow( qt4.QGraphicsView ):
         self.dpi = (pixmap.logicalDpiX(), pixmap.logicalDpiY())
         self.pixmapitem = self.scene.addPixmap(pixmap)
 
+        # whether full screen mode
+        self.isfullscreen = False
+
         # set to be parent's actions
         self.vzactions = None
 
@@ -270,6 +273,9 @@ class PlotWindow( qt4.QGraphicsView ):
         self.clickmode = 'select'
         self.currentclickmode = None
 
+        # wheel zooming accumulator
+        self.sumwheeldelta = 0
+
         # set up redrawing timer
         self.timer = qt4.QTimer(self)
         self.connect( self.timer, qt4.SIGNAL('timeout()'),
@@ -306,9 +312,6 @@ class PlotWindow( qt4.QGraphicsView ):
         # create toolbar in main window (urgh)
         self.createToolbar(parent, menu)
 
-        # make the context menu object
-        self._constructContextMenu()
-
     def hideEvent(self, event):
         """Window closing, so exit rendering threads."""
         self.rendercontrol.exitThreads()
@@ -334,9 +337,10 @@ class PlotWindow( qt4.QGraphicsView ):
         iconsize = setting.settingdb['toolbar_size']
         self.viewtoolbar.setIconSize(qt4.QSize(iconsize, iconsize))
         self.viewtoolbar.hide()
-        parent.addToolBar(qt4.Qt.TopToolBarArea, self.viewtoolbar)
+        if parent:
+            parent.addToolBar(qt4.Qt.TopToolBarArea, self.viewtoolbar)
 
-        if hasattr(parent, 'vzactions'):
+        if parent and hasattr(parent, 'vzactions'):
             # share actions with parent if possible
             # as plot windows can be isolated from mainwindows, we need this
             self.vzactions = actions = parent.vzactions
@@ -395,6 +399,10 @@ class PlotWindow( qt4.QGraphicsView ):
                     a(self, 'Zoom into graph', 'Zoom graph',
                       None,
                       icon='veusz-zoom-graph'),
+                'view.fullscreen':
+                    a(self, 'View plot full screen', 'Full screen',
+                      self.slotFullScreen,
+                      icon='veusz-view-fullscreen', key='Ctrl+F11'),
                 })
 
         if menu:
@@ -406,6 +414,8 @@ class PlotWindow( qt4.QGraphicsView ):
                         'view.zoomheight', 'view.zoompage',
                         '',
                         'view.prevpage', 'view.nextpage',
+                        'view.fullscreen',
+                        '',
                         'view.select', 'view.pick', 'view.zoomgraph',
                         ]),
                 ]
@@ -431,6 +441,7 @@ class PlotWindow( qt4.QGraphicsView ):
         # add items to toolbar
         utils.addToolbarActions(self.viewtoolbar, actions,
                                 ('view.prevpage', 'view.nextpage',
+                                 'view.fullscreen',
                                  'view.select', 'view.pick',
                                  'view.zoomgraph', 'view.zoommenu'))
 
@@ -728,22 +739,24 @@ class PlotWindow( qt4.QGraphicsView ):
     def keyPressEvent(self, event):
         """Keypad motion moves the picker if it has focus"""
         if self.pickeritem.hasFocus():
-            event.accept()
+
             k = event.key()
             if k == qt4.Qt.Key_Left or k == qt4.Qt.Key_Right:
                 # navigate to the previous or next point on the curve
+                event.accept()
                 dir = 'right' if k == qt4.Qt.Key_Right else 'left'
                 ix = self.pickerinfo.index
                 pickinfo = self.pickerinfo.widget.pickIndex(
                     ix, dir, self.painthelper.widgetBounds(
                         self.pickerinfo.widget))
-                if not pickinfo:
-                    # no more points visible in this direction
-                    return
+                if pickinfo:
+                    # more points visible in this direction
+                    self.emitPicked(pickinfo)
+                return
 
-                self.emitPicked(pickinfo)
             elif k == qt4.Qt.Key_Up or k == qt4.Qt.Key_Down:
                 # navigate to the next plot up or down on the screen
+                event.accept()
                 p = self.pickeritem.pos()
 
                 oldw = self.pickerinfo.widget
@@ -779,6 +792,25 @@ class PlotWindow( qt4.QGraphicsView ):
                     # stays repeatable
                     pickinfo.screenpos = (oldx, pickinfo.screenpos[1])
 
+                return
+
+        # handle up-stream
+        qt4.QGraphicsView.keyPressEvent(self, event)
+
+    def wheelEvent(self, event):
+        """For zooming in or moving."""
+
+        if event.modifiers() & qt4.Qt.ControlModifier:
+            self.sumwheeldelta += event.delta()
+            while self.sumwheeldelta <= -120:
+                self.slotViewZoomOut()
+                self.sumwheeldelta += 120
+            while self.sumwheeldelta >= 120:
+                self.slotViewZoomIn()
+                self.sumwheeldelta -= 120
+        else:
+            qt4.QGraphicsView.wheelEvent(self, event)
+
     def locateClickWidget(self, x, y):
         """Work out which widget was clicked, and if necessary send
         a sigWidgetClicked(widget) signal."""
@@ -883,10 +915,17 @@ class PlotWindow( qt4.QGraphicsView ):
         self.setSceneRect(0, 0, bufferpixmap.width(), bufferpixmap.height())
         self.pixmapitem.setPixmap(bufferpixmap)
 
-    def _constructContextMenu(self):
-        """Construct the context menu."""
+    def updatePlotSettings(self):
+        """Update plot window settings from settings."""
+        self.setTimeout(setting.settingdb['plot_updatepolicy'])
+        self.antialias = setting.settingdb['plot_antialias']
+        self.rendercontrol.updateNumberThreads()
+        self.actionForceUpdate()
 
-        menu = self.contextmenu = qt4.QMenu(self)
+    def contextMenuEvent(self, event):
+        """Show context menu."""
+
+        menu = qt4.QMenu(self)
 
         # add some useful entries
         menu.addAction( self.vzactions['view.zoommenu'] )
@@ -898,6 +937,11 @@ class PlotWindow( qt4.QGraphicsView ):
         # force an update now menu item
         menu.addAction('Force update', self.actionForceUpdate)
 
+        if self.isfullscreen:
+            menu.addAction('Close full screen', self.slotFullScreen)
+        else:
+            menu.addAction( self.vzactions['view.fullscreen'] )
+
         # Update policy submenu
         submenu = menu.addMenu('Updates')
         intgrp = qt4.QActionGroup(self)
@@ -918,22 +962,25 @@ class PlotWindow( qt4.QGraphicsView ):
         act.setCheckable(True)
         act.setChecked(self.antialias)
 
-    def updatePlotSettings(self):
-        """Update plot window settings from settings."""
-        self.setTimeout(setting.settingdb['plot_updatepolicy'])
-        self.antialias = setting.settingdb['plot_antialias']
-        self.rendercontrol.updateNumberThreads()
-        self.actionForceUpdate()
-
-    def contextMenuEvent(self, event):
-        """Show context menu."""
-        self.contextmenu.exec_(qt4.QCursor.pos())
+        menu.exec_(qt4.QCursor.pos())
 
     def actionForceUpdate(self):
         """Force an update for the graph."""
         self.docchangeset = -100
         self.checkPlotUpdate()
 
+    def slotFullScreen(self):
+        """Show window full screen or not."""
+        if not self.isfullscreen:
+            self._fullscreenwindow = FullScreenPlotWindow(
+                self.document, self.pagenumber)
+        else:
+            # cheesy way of closing full screen window
+            p = self
+            while p.parent() is not None:
+                p = p.parent()
+            p.close()
+
     def setTimeout(self, interval):
         """Change timer setting without changing save value."""
         self.interval = interval
@@ -1137,3 +1184,55 @@ class PlotWindow( qt4.QGraphicsView ):
                 for control in self.painthelper.states[widget].cgis:
                     graphitem = control.createGraphicsItem()
                     cgg.addToGroup(graphitem)
+
+class FullScreenPlotWindow(qt4.QScrollArea):
+    """Window for showing plot in full-screen mode."""
+
+    def __init__(self, document, pagenumber):
+        qt4.QScrollArea.__init__(self)
+        self.setFrameShape(qt4.QFrame.NoFrame)
+        self.setWidgetResizable(True)
+
+        # window which shows plot
+        self.document = document
+        pw = self.plotwin = PlotWindow(document, None)
+        pw.isfullscreen = True
+        pw.pagenumber = pagenumber
+        self.setWidget(pw)
+        pw.setFocus()
+
+        self.showFullScreen()
+
+        self.toolbar = qt4.QToolBar("Full screen toolbar", self)
+        self.toolbar.addAction(utils.getIcon("kde-window-close"), "Close",
+                               self.close)
+        for a in ('view.zoom11', 'view.zoomin', 'view.zoomout',
+                  'view.zoomwidth', 'view.zoomheight',
+                  'view.zoompage', 'view.prevpage', 'view.nextpage'):
+            self.toolbar.addAction( pw.vzactions[a] )
+        self.toolbar.show()
+
+    def resizeEvent(self, event):
+        """Make zoom fit screen."""
+
+        qt4.QScrollArea.resizeEvent(self, event)
+
+        # size graph to fill screen
+        pagesize = self.document.pageSize(self.plotwin.pagenumber,
+                                          dpi=self.plotwin.dpi)
+        screensize = self.plotwin.size()
+
+        aspectw = screensize.width() * 1. / pagesize[0]
+        aspecth = screensize.height() * 1. / pagesize[1]
+
+        self.plotwin.zoomfactor = min(aspectw, aspecth)
+        self.plotwin.checkPlotUpdate()
+
+    def keyPressEvent(self, event):
+
+        k = event.key()
+        if k == qt4.Qt.Key_Escape:
+            event.accept()
+            self.close()
+            return
+        qt4.QScrollArea.keyPressEvent(self, event)
